import gc
import sys
from typing import List

import pytest
from _pytest.config import ExitCode
from _pytest.monkeypatch import MonkeyPatch
from _pytest.pytester import Pytester


def test_simple_unittest(pytester: Pytester) -> None:
    testpath = pytester.makepyfile(
        """
        import unittest
        class MyTestCase(unittest.TestCase):
            def testpassing(self):
                self.assertEqual('foo', 'foo')
            def test_failing(self):
                self.assertEqual('foo', 'bar')
    """
    )
    reprec = pytester.inline_run(testpath)
    assert reprec.matchreport("testpassing").passed
    assert reprec.matchreport("test_failing").failed


def test_runTest_method(pytester: Pytester) -> None:
    pytester.makepyfile(
        """
        import unittest
        class MyTestCaseWithRunTest(unittest.TestCase):
            def runTest(self):
                self.assertEqual('foo', 'foo')
        class MyTestCaseWithoutRunTest(unittest.TestCase):
            def runTest(self):
                self.assertEqual('foo', 'foo')
            def test_something(self):
                pass
        """
    )
    result = pytester.runpytest("-v")
    result.stdout.fnmatch_lines(
        """
        *MyTestCaseWithRunTest::runTest*
        *MyTestCaseWithoutRunTest::test_something*
        *2 passed*
    """
    )


def test_isclasscheck_issue53(pytester: Pytester) -> None:
    testpath = pytester.makepyfile(
        """
        import unittest
        class _E(object):
            def __getattr__(self, tag):
                pass
        E = _E()
    """
    )
    result = pytester.runpytest(testpath)
    assert result.ret == ExitCode.NO_TESTS_COLLECTED


def test_setup(pytester: Pytester) -> None:
    testpath = pytester.makepyfile(
        """
        import unittest
        class MyTestCase(unittest.TestCase):
            def setUp(self):
                self.foo = 1
            def setup_method(self, method):
                self.foo2 = 1
            def test_both(self):
                self.assertEqual(1, self.foo)
                assert self.foo2 == 1
            def teardown_method(self, method):
                assert 0, "42"

    """
    )
    reprec = pytester.inline_run("-s", testpath)
    assert reprec.matchreport("test_both", when="call").passed
    rep = reprec.matchreport("test_both", when="teardown")
    assert rep.failed and "42" in str(rep.longrepr)


def test_setUpModule(pytester: Pytester) -> None:
    testpath = pytester.makepyfile(
        """
        values = []

        def setUpModule():
            values.append(1)

        def tearDownModule():
            del values[0]

        def test_hello():
            assert values == [1]

        def test_world():
            assert values == [1]
        """
    )
    result = pytester.runpytest(testpath)
    result.stdout.fnmatch_lines(["*2 passed*"])


def test_setUpModule_failing_no_teardown(pytester: Pytester) -> None:
    testpath = pytester.makepyfile(
        """
        values = []

        def setUpModule():
            0/0

        def tearDownModule():
            values.append(1)

        def test_hello():
            pass
    """
    )
    reprec = pytester.inline_run(testpath)
    reprec.assertoutcome(passed=0, failed=1)
    call = reprec.getcalls("pytest_runtest_setup")[0]
    assert not call.item.module.values


def test_new_instances(pytester: Pytester) -> None:
    testpath = pytester.makepyfile(
        """
        import unittest
        class MyTestCase(unittest.TestCase):
            def test_func1(self):
                self.x = 2
            def test_func2(self):
                assert not hasattr(self, 'x')
    """
    )
    reprec = pytester.inline_run(testpath)
    reprec.assertoutcome(passed=2)


def test_function_item_obj_is_instance(pytester: Pytester) -> None:
    """item.obj should be a bound method on unittest.TestCase function items (#5390)."""
    pytester.makeconftest(
        """
        def pytest_runtest_makereport(item, call):
            if call.when == 'call':
                class_ = item.parent.obj
                assert isinstance(item.obj.__self__, class_)
    """
    )
    pytester.makepyfile(
        """
        import unittest

        class Test(unittest.TestCase):
            def test_foo(self):
                pass
    """
    )
    result = pytester.runpytest_inprocess()
    result.stdout.fnmatch_lines(["* 1 passed in*"])


def test_teardown(pytester: Pytester) -> None:
    testpath = pytester.makepyfile(
        """
        import unittest
        class MyTestCase(unittest.TestCase):
            values = []
            def test_one(self):
                pass
            def tearDown(self):
                self.values.append(None)
        class Second(unittest.TestCase):
            def test_check(self):
                self.assertEqual(MyTestCase.values, [None])
    """
    )
    reprec = pytester.inline_run(testpath)
    passed, skipped, failed = reprec.countoutcomes()
    assert failed == 0, failed
    assert passed == 2
    assert passed + skipped + failed == 2


def test_teardown_issue1649(pytester: Pytester) -> None:
    """
    Are TestCase objects cleaned up? Often unittest TestCase objects set
    attributes that are large and expensive during setUp.

    The TestCase will not be cleaned up if the test fails, because it
    would then exist in the stackframe.
    """
    testpath = pytester.makepyfile(
        """
        import unittest
        class TestCaseObjectsShouldBeCleanedUp(unittest.TestCase):
            def setUp(self):
                self.an_expensive_object = 1
            def test_demo(self):
                pass

    """
    )
    pytester.inline_run("-s", testpath)
    gc.collect()
    for obj in gc.get_objects():
        assert type(obj).__name__ != "TestCaseObjectsShouldBeCleanedUp"


def test_unittest_skip_issue148(pytester: Pytester) -> None:
    testpath = pytester.makepyfile(
        """
        import unittest

        @unittest.skip("hello")
        class MyTestCase(unittest.TestCase):
            @classmethod
            def setUpClass(self):
                xxx
            def test_one(self):
                pass
            @classmethod
            def tearDownClass(self):
                xxx
    """
    )
    reprec = pytester.inline_run(testpath)
    reprec.assertoutcome(skipped=1)


def test_method_and_teardown_failing_reporting(pytester: Pytester) -> None:
    pytester.makepyfile(
        """
        import unittest
        class TC(unittest.TestCase):
            def tearDown(self):
                assert 0, "down1"
            def test_method(self):
                assert False, "down2"
    """
    )
    result = pytester.runpytest("-s")
    assert result.ret == 1
    result.stdout.fnmatch_lines(
        [
            "*tearDown*",
            "*assert 0*",
            "*test_method*",
            "*assert False*",
            "*1 failed*1 error*",
        ]
    )


def test_setup_failure_is_shown(pytester: Pytester) -> None:
    pytester.makepyfile(
        """
        import unittest
        import pytest
        class TC(unittest.TestCase):
            def setUp(self):
                assert 0, "down1"
            def test_method(self):
                print("never42")
                xyz
    """
    )
    result = pytester.runpytest("-s")
    assert result.ret == 1
    result.stdout.fnmatch_lines(["*setUp*", "*assert 0*down1*", "*1 failed*"])
    result.stdout.no_fnmatch_line("*never42*")


def test_setup_setUpClass(pytester: Pytester) -> None:
    testpath = pytester.makepyfile(
        """
        import unittest
        import pytest
        class MyTestCase(unittest.TestCase):
            x = 0
            @classmethod
            def setUpClass(cls):
                cls.x += 1
            def test_func1(self):
                assert self.x == 1
            def test_func2(self):
                assert self.x == 1
            @classmethod
            def tearDownClass(cls):
                cls.x -= 1
        def test_teareddown():
            assert MyTestCase.x == 0
    """
    )
    reprec = pytester.inline_run(testpath)
    reprec.assertoutcome(passed=3)


def test_fixtures_setup_setUpClass_issue8394(pytester: Pytester) -> None:
    pytester.makepyfile(
        """
        import unittest
        class MyTestCase(unittest.TestCase):
            @classmethod
            def setUpClass(cls):
                pass
            def test_func1(self):
                pass
            @classmethod
            def tearDownClass(cls):
                pass
    """
    )
    result = pytester.runpytest("--fixtures")
    assert result.ret == 0
    result.stdout.no_fnmatch_line("*no docstring available*")

    result = pytester.runpytest("--fixtures", "-v")
    assert result.ret == 0
    result.stdout.fnmatch_lines(["*no docstring available*"])


def test_setup_class(pytester: Pytester) -> None:
    testpath = pytester.makepyfile(
        """
        import unittest
        import pytest
        class MyTestCase(unittest.TestCase):
            x = 0
            def setup_class(cls):
                cls.x += 1
            def test_func1(self):
                assert self.x == 1
            def test_func2(self):
                assert self.x == 1
            def teardown_class(cls):
                cls.x -= 1
        def test_teareddown():
            assert MyTestCase.x == 0
    """
    )
    reprec = pytester.inline_run(testpath)
    reprec.assertoutcome(passed=3)


@pytest.mark.parametrize("type", ["Error", "Failure"])
def test_testcase_adderrorandfailure_defers(pytester: Pytester, type: str) -> None:
    pytester.makepyfile(
        """
        from unittest import TestCase
        import pytest
        class MyTestCase(TestCase):
            def run(self, result):
                excinfo = pytest.raises(ZeroDivisionError, lambda: 0/0)
                try:
                    result.add%s(self, excinfo._excinfo)
                except KeyboardInterrupt:
                    raise
                except:
                    pytest.fail("add%s should not raise")
            def test_hello(self):
                pass
    """
        % (type, type)
    )
    result = pytester.runpytest()
    result.stdout.no_fnmatch_line("*should not raise*")


@pytest.mark.parametrize("type", ["Error", "Failure"])
def test_testcase_custom_exception_info(pytester: Pytester, type: str) -> None:
    pytester.makepyfile(
        """
        from typing import Generic, TypeVar
        from unittest import TestCase
        import pytest, _pytest._code

        class MyTestCase(TestCase):
            def run(self, result):
                excinfo = pytest.raises(ZeroDivisionError, lambda: 0/0)
                # We fake an incompatible exception info.
                class FakeExceptionInfo(Generic[TypeVar("E")]):
                    def __init__(self, *args, **kwargs):
                        mp.undo()
                        raise TypeError()
                    @classmethod
                    def from_current(cls):
                        return cls()
                    @classmethod
                    def from_exc_info(cls, *args, **kwargs):
                        return cls()
                mp = pytest.MonkeyPatch()
                mp.setattr(_pytest._code, 'ExceptionInfo', FakeExceptionInfo)
                try:
                    excinfo = excinfo._excinfo
                    result.add%(type)s(self, excinfo)
                finally:
                    mp.undo()

            def test_hello(self):
                pass
    """
        % locals()
    )
    result = pytester.runpytest()
    result.stdout.fnmatch_lines(
        [
            "NOTE: Incompatible Exception Representation*",
            "*ZeroDivisionError*",
            "*1 failed*",
        ]
    )


def test_testcase_totally_incompatible_exception_info(pytester: Pytester) -> None:
    import _pytest.unittest

    (item,) = pytester.getitems(
        """
        from unittest import TestCase
        class MyTestCase(TestCase):
            def test_hello(self):
                pass
    """
    )
    assert isinstance(item, _pytest.unittest.TestCaseFunction)
    item.addError(None, 42)  # type: ignore[arg-type]
    excinfo = item._excinfo
    assert excinfo is not None
    assert "ERROR: Unknown Incompatible" in str(excinfo.pop(0).getrepr())


def test_module_level_pytestmark(pytester: Pytester) -> None:
    testpath = pytester.makepyfile(
        """
        import unittest
        import pytest
        pytestmark = pytest.mark.xfail
        class MyTestCase(unittest.TestCase):
            def test_func1(self):
                assert 0
    """
    )
    reprec = pytester.inline_run(testpath, "-s")
    reprec.assertoutcome(skipped=1)


class TestTrialUnittest:
    def setup_class(cls):
        cls.ut = pytest.importorskip("twisted.trial.unittest")
        # on windows trial uses a socket for a reactor and apparently doesn't close it properly
        # https://twistedmatrix.com/trac/ticket/9227
        cls.ignore_unclosed_socket_warning = ("-W", "always")

    def test_trial_testcase_runtest_not_collected(self, pytester: Pytester) -> None:
        pytester.makepyfile(
            """
            from twisted.trial.unittest import TestCase

            class TC(TestCase):
                def test_hello(self):
                    pass
        """
        )
        reprec = pytester.inline_run(*self.ignore_unclosed_socket_warning)
        reprec.assertoutcome(passed=1)
        pytester.makepyfile(
            """
            from twisted.trial.unittest import TestCase

            class TC(TestCase):
                def runTest(self):
                    pass
        """
        )
        reprec = pytester.inline_run(*self.ignore_unclosed_socket_warning)
        reprec.assertoutcome(passed=1)

    def test_trial_exceptions_with_skips(self, pytester: Pytester) -> None:
        pytester.makepyfile(
            """
            from twisted.trial import unittest
            import pytest
            class TC(unittest.TestCase):
                def test_hello(self):
                    pytest.skip("skip_in_method")
                @pytest.mark.skipif("sys.version_info != 1")
                def test_hello2(self):
                    pass
                @pytest.mark.xfail(reason="iwanto")
                def test_hello3(self):
                    assert 0
                def test_hello4(self):
                    pytest.xfail("i2wanto")
                def test_trial_skip(self):
                    pass
                test_trial_skip.skip = "trialselfskip"

                def test_trial_todo(self):
                    assert 0
                test_trial_todo.todo = "mytodo"

                def test_trial_todo_success(self):
                    pass
                test_trial_todo_success.todo = "mytodo"

            class TC2(unittest.TestCase):
                def setup_class(cls):
                    pytest.skip("skip_in_setup_class")
                def test_method(self):
                    pass
        """
        )
        result = pytester.runpytest("-rxs", *self.ignore_unclosed_socket_warning)
        result.stdout.fnmatch_lines_random(
            [
                "*XFAIL*test_trial_todo*",
                "*trialselfskip*",
                "*skip_in_setup_class*",
                "*iwanto*",
                "*i2wanto*",
                "*sys.version_info*",
                "*skip_in_method*",
                "*1 failed*4 skipped*3 xfailed*",
            ]
        )
        assert result.ret == 1

    def test_trial_error(self, pytester: Pytester) -> None:
        pytester.makepyfile(
            """
            from twisted.trial.unittest import TestCase
            from twisted.internet.defer import Deferred
            from twisted.internet import reactor

            class TC(TestCase):
                def test_one(self):
                    crash

                def test_two(self):
                    def f(_):
                        crash

                    d = Deferred()
                    d.addCallback(f)
                    reactor.callLater(0.3, d.callback, None)
                    return d

                def test_three(self):
                    def f():
                        pass # will never get called
                    reactor.callLater(0.3, f)
                # will crash at teardown

                def test_four(self):
                    def f(_):
                        reactor.callLater(0.3, f)
                        crash

                    d = Deferred()
                    d.addCallback(f)
                    reactor.callLater(0.3, d.callback, None)
                    return d
                # will crash both at test time and at teardown
        """
        )
        result = pytester.runpytest(
            "-vv", "-oconsole_output_style=classic", "-W", "ignore::DeprecationWarning"
        )
        result.stdout.fnmatch_lines(
            [
                "test_trial_error.py::TC::test_four FAILED",
                "test_trial_error.py::TC::test_four ERROR",
                "test_trial_error.py::TC::test_one FAILED",
                "test_trial_error.py::TC::test_three FAILED",
                "test_trial_error.py::TC::test_two FAILED",
                "*ERRORS*",
                "*_ ERROR at teardown of TC.test_four _*",
                "*DelayedCalls*",
                "*= FAILURES =*",
                "*_ TC.test_four _*",
                "*NameError*crash*",
                "*_ TC.test_one _*",
                "*NameError*crash*",
                "*_ TC.test_three _*",
                "*DelayedCalls*",
                "*_ TC.test_two _*",
                "*NameError*crash*",
                "*= 4 failed, 1 error in *",
            ]
        )

    def test_trial_pdb(self, pytester: Pytester) -> None:
        p = pytester.makepyfile(
            """
            from twisted.trial import unittest
            import pytest
            class TC(unittest.TestCase):
                def test_hello(self):
                    assert 0, "hellopdb"
        """
        )
        child = pytester.spawn_pytest(str(p))
        child.expect("hellopdb")
        child.sendeof()

    def test_trial_testcase_skip_property(self, pytester: Pytester) -> None:
        testpath = pytester.makepyfile(
            """
            from twisted.trial import unittest
            class MyTestCase(unittest.TestCase):
                skip = 'dont run'
                def test_func(self):
                    pass
            """
        )
        reprec = pytester.inline_run(testpath, "-s")
        reprec.assertoutcome(skipped=1)

    def test_trial_testfunction_skip_property(self, pytester: Pytester) -> None:
        testpath = pytester.makepyfile(
            """
            from twisted.trial import unittest
            class MyTestCase(unittest.TestCase):
                def test_func(self):
                    pass
                test_func.skip = 'dont run'
            """
        )
        reprec = pytester.inline_run(testpath, "-s")
        reprec.assertoutcome(skipped=1)

    def test_trial_testcase_todo_property(self, pytester: Pytester) -> None:
        testpath = pytester.makepyfile(
            """
            from twisted.trial import unittest
            class MyTestCase(unittest.TestCase):
                todo = 'dont run'
                def test_func(self):
                    assert 0
            """
        )
        reprec = pytester.inline_run(testpath, "-s")
        reprec.assertoutcome(skipped=1)

    def test_trial_testfunction_todo_property(self, pytester: Pytester) -> None:
        testpath = pytester.makepyfile(
            """
            from twisted.trial import unittest
            class MyTestCase(unittest.TestCase):
                def test_func(self):
                    assert 0
                test_func.todo = 'dont run'
            """
        )
        reprec = pytester.inline_run(
            testpath, "-s", *self.ignore_unclosed_socket_warning
        )
        reprec.assertoutcome(skipped=1)


def test_djangolike_testcase(pytester: Pytester) -> None:
    # contributed from Morten Breekevold
    pytester.makepyfile(
        """
        from unittest import TestCase, main

        class DjangoLikeTestCase(TestCase):

            def setUp(self):
                print("setUp()")

            def test_presetup_has_been_run(self):
                print("test_thing()")
                self.assertTrue(hasattr(self, 'was_presetup'))

            def tearDown(self):
                print("tearDown()")

            def __call__(self, result=None):
                try:
                    self._pre_setup()
                except (KeyboardInterrupt, SystemExit):
                    raise
                except Exception:
                    import sys
                    result.addError(self, sys.exc_info())
                    return
                super(DjangoLikeTestCase, self).__call__(result)
                try:
                    self._post_teardown()
                except (KeyboardInterrupt, SystemExit):
                    raise
                except Exception:
                    import sys
                    result.addError(self, sys.exc_info())
                    return

            def _pre_setup(self):
                print("_pre_setup()")
                self.was_presetup = True

            def _post_teardown(self):
                print("_post_teardown()")
    """
    )
    result = pytester.runpytest("-s")
    assert result.ret == 0
    result.stdout.fnmatch_lines(
        [
            "*_pre_setup()*",
            "*setUp()*",
            "*test_thing()*",
            "*tearDown()*",
            "*_post_teardown()*",
        ]
    )


def test_unittest_not_shown_in_traceback(pytester: Pytester) -> None:
    pytester.makepyfile(
        """
        import unittest
        class t(unittest.TestCase):
            def test_hello(self):
                x = 3
                self.assertEqual(x, 4)
    """
    )
    res = pytester.runpytest()
    res.stdout.no_fnmatch_line("*failUnlessEqual*")


def test_unorderable_types(pytester: Pytester) -> None:
    pytester.makepyfile(
        """
        import unittest
        class TestJoinEmpty(unittest.TestCase):
            pass

        def make_test():
            class Test(unittest.TestCase):
                pass
            Test.__name__ = "TestFoo"
            return Test
        TestFoo = make_test()
    """
    )
    result = pytester.runpytest()
    result.stdout.no_fnmatch_line("*TypeError*")
    assert result.ret == ExitCode.NO_TESTS_COLLECTED


def test_unittest_typerror_traceback(pytester: Pytester) -> None:
    pytester.makepyfile(
        """
        import unittest
        class TestJoinEmpty(unittest.TestCase):
            def test_hello(self, arg1):
                pass
    """
    )
    result = pytester.runpytest()
    assert "TypeError" in result.stdout.str()
    assert result.ret == 1


@pytest.mark.parametrize("runner", ["pytest", "unittest"])
def test_unittest_expected_failure_for_failing_test_is_xfail(
    pytester: Pytester, runner
) -> None:
    script = pytester.makepyfile(
        """
        import unittest
        class MyTestCase(unittest.TestCase):
            @unittest.expectedFailure
            def test_failing_test_is_xfail(self):
                assert False
        if __name__ == '__main__':
            unittest.main()
    """
    )
    if runner == "pytest":
        result = pytester.runpytest("-rxX")
        result.stdout.fnmatch_lines(
            ["*XFAIL*MyTestCase*test_failing_test_is_xfail*", "*1 xfailed*"]
        )
    else:
        result = pytester.runpython(script)
        result.stderr.fnmatch_lines(["*1 test in*", "*OK*(expected failures=1)*"])
    assert result.ret == 0


@pytest.mark.parametrize("runner", ["pytest", "unittest"])
def test_unittest_expected_failure_for_passing_test_is_fail(
    pytester: Pytester,
    runner: str,
) -> None:
    script = pytester.makepyfile(
        """
        import unittest
        class MyTestCase(unittest.TestCase):
            @unittest.expectedFailure
            def test_passing_test_is_fail(self):
                assert True
        if __name__ == '__main__':
            unittest.main()
    """
    )

    if runner == "pytest":
        result = pytester.runpytest("-rxX")
        result.stdout.fnmatch_lines(
            [
                "*MyTestCase*test_passing_test_is_fail*",
                "Unexpected success",
                "*1 failed*",
            ]
        )
    else:
        result = pytester.runpython(script)
        result.stderr.fnmatch_lines(["*1 test in*", "*(unexpected successes=1)*"])

    assert result.ret == 1


@pytest.mark.parametrize("stmt", ["return", "yield"])
def test_unittest_setup_interaction(pytester: Pytester, stmt: str) -> None:
    pytester.makepyfile(
        """
        import unittest
        import pytest
        class MyTestCase(unittest.TestCase):
            @pytest.fixture(scope="class", autouse=True)
            def perclass(self, request):
                request.cls.hello = "world"
                {stmt}
            @pytest.fixture(scope="function", autouse=True)
            def perfunction(self, request):
                request.instance.funcname = request.function.__name__
                {stmt}

            def test_method1(self):
                assert self.funcname == "test_method1"
                assert self.hello == "world"

            def test_method2(self):
                assert self.funcname == "test_method2"

            def test_classattr(self):
                assert self.__class__.hello == "world"
    """.format(
            stmt=stmt
        )
    )
    result = pytester.runpytest()
    result.stdout.fnmatch_lines(["*3 passed*"])


def test_non_unittest_no_setupclass_support(pytester: Pytester) -> None:
    testpath = pytester.makepyfile(
        """
        class TestFoo(object):
            x = 0

            @classmethod
            def setUpClass(cls):
                cls.x = 1

            def test_method1(self):
                assert self.x == 0

            @classmethod
            def tearDownClass(cls):
                cls.x = 1

        def test_not_teareddown():
            assert TestFoo.x == 0

    """
    )
    reprec = pytester.inline_run(testpath)
    reprec.assertoutcome(passed=2)


def test_no_teardown_if_setupclass_failed(pytester: Pytester) -> None:
    testpath = pytester.makepyfile(
        """
        import unittest

        class MyTestCase(unittest.TestCase):
            x = 0

            @classmethod
            def setUpClass(cls):
                cls.x = 1
                assert False

            def test_func1(self):
                cls.x = 10

            @classmethod
            def tearDownClass(cls):
                cls.x = 100

        def test_notTornDown():
            assert MyTestCase.x == 1
    """
    )
    reprec = pytester.inline_run(testpath)
    reprec.assertoutcome(passed=1, failed=1)


def test_cleanup_functions(pytester: Pytester) -> None:
    """Ensure functions added with addCleanup are always called after each test ends (#6947)"""
    pytester.makepyfile(
        """
        import unittest

        cleanups = []

        class Test(unittest.TestCase):

            def test_func_1(self):
                self.addCleanup(cleanups.append, "test_func_1")

            def test_func_2(self):
                self.addCleanup(cleanups.append, "test_func_2")
                assert 0

            def test_func_3_check_cleanups(self):
                assert cleanups == ["test_func_1", "test_func_2"]
    """
    )
    result = pytester.runpytest("-v")
    result.stdout.fnmatch_lines(
        [
            "*::test_func_1 PASSED *",
            "*::test_func_2 FAILED *",
            "*::test_func_3_check_cleanups PASSED *",
        ]
    )


def test_issue333_result_clearing(pytester: Pytester) -> None:
    pytester.makeconftest(
        """
        import pytest
        @pytest.hookimpl(hookwrapper=True)
        def pytest_runtest_call(item):
            yield
            assert 0
    """
    )
    pytester.makepyfile(
        """
        import unittest
        class TestIt(unittest.TestCase):
            def test_func(self):
                0/0
    """
    )

    reprec = pytester.inline_run()
    reprec.assertoutcome(failed=1)


def test_unittest_raise_skip_issue748(pytester: Pytester) -> None:
    pytester.makepyfile(
        test_foo="""
        import unittest

        class MyTestCase(unittest.TestCase):
            def test_one(self):
                raise unittest.SkipTest('skipping due to reasons')
    """
    )
    result = pytester.runpytest("-v", "-rs")
    result.stdout.fnmatch_lines(
        """
        *SKIP*[1]*test_foo.py*skipping due to reasons*
        *1 skipped*
    """
    )


def test_unittest_skip_issue1169(pytester: Pytester) -> None:
    pytester.makepyfile(
        test_foo="""
        import unittest

        class MyTestCase(unittest.TestCase):
            @unittest.skip("skipping due to reasons")
            def test_skip(self):
                 self.fail()
        """
    )
    result = pytester.runpytest("-v", "-rs")
    result.stdout.fnmatch_lines(
        """
        *SKIP*[1]*skipping due to reasons*
        *1 skipped*
    """
    )


def test_class_method_containing_test_issue1558(pytester: Pytester) -> None:
    pytester.makepyfile(
        test_foo="""
        import unittest

        class MyTestCase(unittest.TestCase):
            def test_should_run(self):
                pass
            def test_should_not_run(self):
                pass
            test_should_not_run.__test__ = False
    """
    )
    reprec = pytester.inline_run()
    reprec.assertoutcome(passed=1)


@pytest.mark.parametrize("base", ["builtins.object", "unittest.TestCase"])
def test_usefixtures_marker_on_unittest(base, pytester: Pytester) -> None:
    """#3498"""
    module = base.rsplit(".", 1)[0]
    pytest.importorskip(module)
    pytester.makepyfile(
        conftest="""
        import pytest

        @pytest.fixture(scope='function')
        def fixture1(request, monkeypatch):
            monkeypatch.setattr(request.instance, 'fixture1', True )


        @pytest.fixture(scope='function')
        def fixture2(request, monkeypatch):
            monkeypatch.setattr(request.instance, 'fixture2', True )

        def node_and_marks(item):
            print(item.nodeid)
            for mark in item.iter_markers():
                print("  ", mark)

        @pytest.fixture(autouse=True)
        def my_marks(request):
            node_and_marks(request.node)

        def pytest_collection_modifyitems(items):
            for item in items:
               node_and_marks(item)

        """
    )

    pytester.makepyfile(
        """
        import pytest
        import {module}

        class Tests({base}):
            fixture1 = False
            fixture2 = False

            @pytest.mark.usefixtures("fixture1")
            def test_one(self):
                assert self.fixture1
                assert not self.fixture2

            @pytest.mark.usefixtures("fixture1", "fixture2")
            def test_two(self):
                assert self.fixture1
                assert self.fixture2


    """.format(
            module=module, base=base
        )
    )

    result = pytester.runpytest("-s")
    result.assert_outcomes(passed=2)


def test_testcase_handles_init_exceptions(pytester: Pytester) -> None:
    """
    Regression test to make sure exceptions in the __init__ method are bubbled up correctly.
    See https://github.com/pytest-dev/pytest/issues/3788
    """
    pytester.makepyfile(
        """
        from unittest import TestCase
        import pytest
        class MyTestCase(TestCase):
            def __init__(self, *args, **kwargs):
                raise Exception("should raise this exception")
            def test_hello(self):
                pass
    """
    )
    result = pytester.runpytest()
    assert "should raise this exception" in result.stdout.str()
    result.stdout.no_fnmatch_line("*ERROR at teardown of MyTestCase.test_hello*")


def test_error_message_with_parametrized_fixtures(pytester: Pytester) -> None:
    pytester.copy_example("unittest/test_parametrized_fixture_error_message.py")
    result = pytester.runpytest()
    result.stdout.fnmatch_lines(
        [
            "*test_two does not support fixtures*",
            "*TestSomethingElse::test_two",
            "*Function type: TestCaseFunction",
        ]
    )


@pytest.mark.parametrize(
    "test_name, expected_outcome",
    [
        ("test_setup_skip.py", "1 skipped"),
        ("test_setup_skip_class.py", "1 skipped"),
        ("test_setup_skip_module.py", "1 error"),
    ],
)
def test_setup_inheritance_skipping(
    pytester: Pytester, test_name, expected_outcome
) -> None:
    """Issue #4700"""
    pytester.copy_example(f"unittest/{test_name}")
    result = pytester.runpytest()
    result.stdout.fnmatch_lines([f"* {expected_outcome} in *"])


def test_BdbQuit(pytester: Pytester) -> None:
    pytester.makepyfile(
        test_foo="""
        import unittest

        class MyTestCase(unittest.TestCase):
            def test_bdbquit(self):
                import bdb
                raise bdb.BdbQuit()

            def test_should_not_run(self):
                pass
    """
    )
    reprec = pytester.inline_run()
    reprec.assertoutcome(failed=1, passed=1)


def test_exit_outcome(pytester: Pytester) -> None:
    pytester.makepyfile(
        test_foo="""
        import pytest
        import unittest

        class MyTestCase(unittest.TestCase):
            def test_exit_outcome(self):
                pytest.exit("pytest_exit called")

            def test_should_not_run(self):
                pass
    """
    )
    result = pytester.runpytest()
    result.stdout.fnmatch_lines(["*Exit: pytest_exit called*", "*= no tests ran in *"])


def test_trace(pytester: Pytester, monkeypatch: MonkeyPatch) -> None:
    calls = []

    def check_call(*args, **kwargs):
        calls.append((args, kwargs))
        assert args == ("runcall",)

        class _pdb:
            def runcall(*args, **kwargs):
                calls.append((args, kwargs))

        return _pdb

    monkeypatch.setattr("_pytest.debugging.pytestPDB._init_pdb", check_call)

    p1 = pytester.makepyfile(
        """
        import unittest

        class MyTestCase(unittest.TestCase):
            def test(self):
                self.assertEqual('foo', 'foo')
    """
    )
    result = pytester.runpytest("--trace", str(p1))
    assert len(calls) == 2
    assert result.ret == 0


def test_pdb_teardown_called(pytester: Pytester, monkeypatch: MonkeyPatch) -> None:
    """Ensure tearDown() is always called when --pdb is given in the command-line.

    We delay the normal tearDown() calls when --pdb is given, so this ensures we are calling
    tearDown() eventually to avoid memory leaks when using --pdb.
    """
    teardowns: List[str] = []
    monkeypatch.setattr(
        pytest, "test_pdb_teardown_called_teardowns", teardowns, raising=False
    )

    pytester.makepyfile(
        """
        import unittest
        import pytest

        class MyTestCase(unittest.TestCase):

            def tearDown(self):
                pytest.test_pdb_teardown_called_teardowns.append(self.id())

            def test_1(self):
                pass
            def test_2(self):
                pass
    """
    )
    result = pytester.runpytest_inprocess("--pdb")
    result.stdout.fnmatch_lines("* 2 passed in *")
    assert teardowns == [
        "test_pdb_teardown_called.MyTestCase.test_1",
        "test_pdb_teardown_called.MyTestCase.test_2",
    ]


@pytest.mark.parametrize("mark", ["@unittest.skip", "@pytest.mark.skip"])
def test_pdb_teardown_skipped(
    pytester: Pytester, monkeypatch: MonkeyPatch, mark: str
) -> None:
    """With --pdb, setUp and tearDown should not be called for skipped tests."""
    tracked: List[str] = []
    monkeypatch.setattr(pytest, "test_pdb_teardown_skipped", tracked, raising=False)

    pytester.makepyfile(
        """
        import unittest
        import pytest

        class MyTestCase(unittest.TestCase):

            def setUp(self):
                pytest.test_pdb_teardown_skipped.append("setUp:" + self.id())

            def tearDown(self):
                pytest.test_pdb_teardown_skipped.append("tearDown:" + self.id())

            {mark}("skipped for reasons")
            def test_1(self):
                pass

    """.format(
            mark=mark
        )
    )
    result = pytester.runpytest_inprocess("--pdb")
    result.stdout.fnmatch_lines("* 1 skipped in *")
    assert tracked == []


def test_async_support(pytester: Pytester) -> None:
    pytest.importorskip("unittest.async_case")

    pytester.copy_example("unittest/test_unittest_asyncio.py")
    reprec = pytester.inline_run()
    reprec.assertoutcome(failed=1, passed=2)


def test_asynctest_support(pytester: Pytester) -> None:
    """Check asynctest support (#7110)"""
    pytest.importorskip("asynctest")

    pytester.copy_example("unittest/test_unittest_asynctest.py")
    reprec = pytester.inline_run()
    reprec.assertoutcome(failed=1, passed=2)


def test_plain_unittest_does_not_support_async(pytester: Pytester) -> None:
    """Async functions in plain unittest.TestCase subclasses are not supported without plugins.

    This test exists here to avoid introducing this support by accident, leading users
    to expect that it works, rather than doing so intentionally as a feature.

    See https://github.com/pytest-dev/pytest-asyncio/issues/180 for more context.
    """
    pytester.copy_example("unittest/test_unittest_plain_async.py")
    result = pytester.runpytest_subprocess()
    if hasattr(sys, "pypy_version_info"):
        # in PyPy we can't reliable get the warning about the coroutine not being awaited,
        # because it depends on the coroutine being garbage collected; given that
        # we are running in a subprocess, that's difficult to enforce
        expected_lines = ["*1 passed*"]
    else:
        expected_lines = [
            "*RuntimeWarning: coroutine * was never awaited",
            "*1 passed*",
        ]
    result.stdout.fnmatch_lines(expected_lines)


@pytest.mark.skipif(
    sys.version_info < (3, 8), reason="Feature introduced in Python 3.8"
)
def test_do_class_cleanups_on_success(pytester: Pytester) -> None:
    testpath = pytester.makepyfile(
        """
        import unittest
        class MyTestCase(unittest.TestCase):
            values = []
            @classmethod
            def setUpClass(cls):
                def cleanup():
                    cls.values.append(1)
                cls.addClassCleanup(cleanup)
            def test_one(self):
                pass
            def test_two(self):
                pass
        def test_cleanup_called_exactly_once():
            assert MyTestCase.values == [1]
    """
    )
    reprec = pytester.inline_run(testpath)
    passed, skipped, failed = reprec.countoutcomes()
    assert failed == 0
    assert passed == 3


@pytest.mark.skipif(
    sys.version_info < (3, 8), reason="Feature introduced in Python 3.8"
)
def test_do_class_cleanups_on_setupclass_failure(pytester: Pytester) -> None:
    testpath = pytester.makepyfile(
        """
        import unittest
        class MyTestCase(unittest.TestCase):
            values = []
            @classmethod
            def setUpClass(cls):
                def cleanup():
                    cls.values.append(1)
                cls.addClassCleanup(cleanup)
                assert False
            def test_one(self):
                pass
        def test_cleanup_called_exactly_once():
            assert MyTestCase.values == [1]
    """
    )
    reprec = pytester.inline_run(testpath)
    passed, skipped, failed = reprec.countoutcomes()
    assert failed == 1
    assert passed == 1


@pytest.mark.skipif(
    sys.version_info < (3, 8), reason="Feature introduced in Python 3.8"
)
def test_do_class_cleanups_on_teardownclass_failure(pytester: Pytester) -> None:
    testpath = pytester.makepyfile(
        """
        import unittest
        class MyTestCase(unittest.TestCase):
            values = []
            @classmethod
            def setUpClass(cls):
                def cleanup():
                    cls.values.append(1)
                cls.addClassCleanup(cleanup)
            @classmethod
            def tearDownClass(cls):
                assert False
            def test_one(self):
                pass
            def test_two(self):
                pass
        def test_cleanup_called_exactly_once():
            assert MyTestCase.values == [1]
    """
    )
    reprec = pytester.inline_run(testpath)
    passed, skipped, failed = reprec.countoutcomes()
    assert passed == 3


def test_do_cleanups_on_success(pytester: Pytester) -> None:
    testpath = pytester.makepyfile(
        """
        import unittest
        class MyTestCase(unittest.TestCase):
            values = []
            def setUp(self):
                def cleanup():
                    self.values.append(1)
                self.addCleanup(cleanup)
            def test_one(self):
                pass
            def test_two(self):
                pass
        def test_cleanup_called_the_right_number_of_times():
            assert MyTestCase.values == [1, 1]
    """
    )
    reprec = pytester.inline_run(testpath)
    passed, skipped, failed = reprec.countoutcomes()
    assert failed == 0
    assert passed == 3


def test_do_cleanups_on_setup_failure(pytester: Pytester) -> None:
    testpath = pytester.makepyfile(
        """
        import unittest
        class MyTestCase(unittest.TestCase):
            values = []
            def setUp(self):
                def cleanup():
                    self.values.append(1)
                self.addCleanup(cleanup)
                assert False
            def test_one(self):
                pass
            def test_two(self):
                pass
        def test_cleanup_called_the_right_number_of_times():
            assert MyTestCase.values == [1, 1]
    """
    )
    reprec = pytester.inline_run(testpath)
    passed, skipped, failed = reprec.countoutcomes()
    assert failed == 2
    assert passed == 1


def test_do_cleanups_on_teardown_failure(pytester: Pytester) -> None:
    testpath = pytester.makepyfile(
        """
        import unittest
        class MyTestCase(unittest.TestCase):
            values = []
            def setUp(self):
                def cleanup():
                    self.values.append(1)
                self.addCleanup(cleanup)
            def tearDown(self):
                assert False
            def test_one(self):
                pass
            def test_two(self):
                pass
        def test_cleanup_called_the_right_number_of_times():
            assert MyTestCase.values == [1, 1]
    """
    )
    reprec = pytester.inline_run(testpath)
    passed, skipped, failed = reprec.countoutcomes()
    assert failed == 2
    assert passed == 1


def test_traceback_pruning(pytester: Pytester) -> None:
    """Regression test for #9610 - doesn't crash during traceback pruning."""
    pytester.makepyfile(
        """
        import unittest

        class MyTestCase(unittest.TestCase):
            def __init__(self, test_method):
                unittest.TestCase.__init__(self, test_method)

        class TestIt(MyTestCase):
            @classmethod
            def tearDownClass(cls) -> None:
                assert False

            def test_it(self):
                pass
        """
    )
    reprec = pytester.inline_run()
    passed, skipped, failed = reprec.countoutcomes()
    assert passed == 1
    assert failed == 1
    assert reprec.ret == 1



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import os
import sys

import mozunit
import pytest

from mozterm import NullTerminal, Terminal


def test_terminal():
    blessed = pytest.importorskip("blessed")
    term = Terminal()
    assert isinstance(term, blessed.Terminal)

    term = Terminal(disable_styling=True)
    assert isinstance(term, NullTerminal)


def test_null_terminal():
    term = NullTerminal()
    assert term.red("foo") == "foo"
    assert term.red == ""
    assert term.color(1) == ""
    assert term.number_of_colors == 0
    assert term.width == 0
    assert term.height == 0
    assert term.is_a_tty == os.isatty(sys.stdout.fileno())


if __name__ == "__main__":
    mozunit.main()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this file,
# You can obtain one at http://mozilla.org/MPL/2.0/.

from setuptools import find_packages, setup

PACKAGE_NAME = "mozlog"
PACKAGE_VERSION = "8.0.0"
DEPS = [
    "blessed>=1.19.1",
    "mozterm",
    "mozfile",
    "six >= 1.13.0",
]


setup(
    name=PACKAGE_NAME,
    version=PACKAGE_VERSION,
    description="Robust log handling specialized for logging in the Mozilla universe",
    long_description="see https://firefox-source-docs.mozilla.org/mozbase/index.html",
    author="Mozilla Automation and Testing Team",
    author_email="tools@lists.mozilla.org",
    url="https://wiki.mozilla.org/Auto-tools/Projects/Mozbase",
    license="Mozilla Public License 2.0 (MPL 2.0)",
    packages=find_packages(),
    zip_safe=False,
    install_requires=DEPS,
    tests_require=["mozfile"],
    platforms=["Any"],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.5",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    package_data={"mozlog": ["formatters/html/main.js", "formatters/html/style.css"]},
    entry_points={"console_scripts": ["structlog = mozlog.scripts:main"]},
)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this file,
# You can obtain one at http://mozilla.org/MPL/2.0/.

from .intel_power_gadget import (
    IPGEmptyFileError,
    IPGMissingOutputFileError,
    IPGTimeoutError,
    IPGUnknownValueTypeError,
)
from .mozpower import MissingProcessorInfoError, MozPower, OsCpuComboMissingError
from .powerbase import IPGExecutableMissingError, PlatformUnsupportedError

__all__ = [
    "IPGEmptyFileError",
    "IPGExecutableMissingError",
    "IPGMissingOutputFileError",
    "IPGTimeoutError",
    "IPGUnknownValueTypeError",
    "MissingProcessorInfoError",
    "MozPower",
    "OsCpuComboMissingError",
    "PlatformUnsupportedError",
]



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from mach.decorators import Command


@Command("cmd_foo", category="testing", conditions=["invalid"])
def run_foo(command_context):
    pass



import re
import sys
from typing import List

import pytest
from _pytest.monkeypatch import MonkeyPatch
from _pytest.pytester import Pytester

if sys.gettrace():

    @pytest.fixture(autouse=True)
    def restore_tracing():
        """Restore tracing function (when run with Coverage.py).

        https://bugs.python.org/issue37011
        """
        orig_trace = sys.gettrace()
        yield
        if sys.gettrace() != orig_trace:
            sys.settrace(orig_trace)


@pytest.hookimpl(hookwrapper=True, tryfirst=True)
def pytest_collection_modifyitems(items):
    """Prefer faster tests.

    Use a hookwrapper to do this in the beginning, so e.g. --ff still works
    correctly.
    """
    fast_items = []
    slow_items = []
    slowest_items = []
    neutral_items = []

    spawn_names = {"spawn_pytest", "spawn"}

    for item in items:
        try:
            fixtures = item.fixturenames
        except AttributeError:
            # doctest at least
            # (https://github.com/pytest-dev/pytest/issues/5070)
            neutral_items.append(item)
        else:
            if "pytester" in fixtures:
                co_names = item.function.__code__.co_names
                if spawn_names.intersection(co_names):
                    item.add_marker(pytest.mark.uses_pexpect)
                    slowest_items.append(item)
                elif "runpytest_subprocess" in co_names:
                    slowest_items.append(item)
                else:
                    slow_items.append(item)
                item.add_marker(pytest.mark.slow)
            else:
                marker = item.get_closest_marker("slow")
                if marker:
                    slowest_items.append(item)
                else:
                    fast_items.append(item)

    items[:] = fast_items + neutral_items + slow_items + slowest_items

    yield


@pytest.fixture
def tw_mock():
    """Returns a mock terminal writer"""

    class TWMock:
        WRITE = object()

        def __init__(self):
            self.lines = []
            self.is_writing = False

        def sep(self, sep, line=None):
            self.lines.append((sep, line))

        def write(self, msg, **kw):
            self.lines.append((TWMock.WRITE, msg))

        def _write_source(self, lines, indents=()):
            if not indents:
                indents = [""] * len(lines)
            for indent, line in zip(indents, lines):
                self.line(indent + line)

        def line(self, line, **kw):
            self.lines.append(line)

        def markup(self, text, **kw):
            return text

        def get_write_msg(self, idx):
            flag, msg = self.lines[idx]
            assert flag == TWMock.WRITE
            return msg

        fullwidth = 80

    return TWMock()


@pytest.fixture
def dummy_yaml_custom_test(pytester: Pytester):
    """Writes a conftest file that collects and executes a dummy yaml test.

    Taken from the docs, but stripped down to the bare minimum, useful for
    tests which needs custom items collected.
    """
    pytester.makeconftest(
        """
        import pytest

        def pytest_collect_file(parent, file_path):
            if file_path.suffix == ".yaml" and file_path.name.startswith("test"):
                return YamlFile.from_parent(path=file_path, parent=parent)

        class YamlFile(pytest.File):
            def collect(self):
                yield YamlItem.from_parent(name=self.path.name, parent=self)

        class YamlItem(pytest.Item):
            def runtest(self):
                pass
    """
    )
    pytester.makefile(".yaml", test1="")


@pytest.fixture
def pytester(pytester: Pytester, monkeypatch: MonkeyPatch) -> Pytester:
    monkeypatch.setenv("PYTEST_DISABLE_PLUGIN_AUTOLOAD", "1")
    return pytester


@pytest.fixture(scope="session")
def color_mapping():
    """Returns a utility class which can replace keys in strings in the form "{NAME}"
    by their equivalent ASCII codes in the terminal.

    Used by tests which check the actual colors output by pytest.
    """

    class ColorMapping:
        COLORS = {
            "red": "\x1b[31m",
            "green": "\x1b[32m",
            "yellow": "\x1b[33m",
            "bold": "\x1b[1m",
            "reset": "\x1b[0m",
            "kw": "\x1b[94m",
            "hl-reset": "\x1b[39;49;00m",
            "function": "\x1b[92m",
            "number": "\x1b[94m",
            "str": "\x1b[33m",
            "print": "\x1b[96m",
        }
        RE_COLORS = {k: re.escape(v) for k, v in COLORS.items()}

        @classmethod
        def format(cls, lines: List[str]) -> List[str]:
            """Straightforward replacement of color names to their ASCII codes."""
            return [line.format(**cls.COLORS) for line in lines]

        @classmethod
        def format_for_fnmatch(cls, lines: List[str]) -> List[str]:
            """Replace color names for use with LineMatcher.fnmatch_lines"""
            return [line.format(**cls.COLORS).replace("[", "[[]") for line in lines]

        @classmethod
        def format_for_rematch(cls, lines: List[str]) -> List[str]:
            """Replace color names for use with LineMatcher.re_match_lines"""
            return [line.format(**cls.RE_COLORS) for line in lines]

    return ColorMapping


@pytest.fixture
def mock_timing(monkeypatch: MonkeyPatch):
    """Mocks _pytest.timing with a known object that can be used to control timing in tests
    deterministically.

    pytest itself should always use functions from `_pytest.timing` instead of `time` directly.

    This then allows us more control over time during testing, if testing code also
    uses `_pytest.timing` functions.

    Time is static, and only advances through `sleep` calls, thus tests might sleep over large
    numbers and obtain accurate time() calls at the end, making tests reliable and instant.
    """
    import attr

    @attr.s
    class MockTiming:

        _current_time = attr.ib(default=1590150050.0)

        def sleep(self, seconds):
            self._current_time += seconds

        def time(self):
            return self._current_time

        def patch(self):
            from _pytest import timing

            monkeypatch.setattr(timing, "sleep", self.sleep)
            monkeypatch.setattr(timing, "time", self.time)
            monkeypatch.setattr(timing, "perf_counter", self.time)

    result = MockTiming()
    result.patch()
    return result



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import os

from setuptools import find_packages, setup

PACKAGE_VERSION = "0.1"

THIS_DIR = os.path.dirname(os.path.realpath(__name__))


def read(*parts):
    with open(os.path.join(THIS_DIR, *parts)) as f:
        return f.read()


setup(
    name="telemetry-harness",
    version=PACKAGE_VERSION,
    description=(
        "Custom Marionette runner classes and entry scripts for "
        "Telemetry specific Marionette tests."
    ),
    classifiers=[
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    keywords="mozilla",
    author="Firefox Test Engineering Team",
    author_email="firefox-test-engineering@mozilla.org",
    url="https://developer.mozilla.org/en-US/docs/Mozilla/QA/telemetry_harness",
    license="MPL 2.0",
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=read("requirements.txt").splitlines(),
    entry_points="""
        [console_scripts]
        telemetry-harness = telemetry_harness.runtests:cli
    """,
)



#!/usr/bin/env python

import os
import shutil
import tempfile
import unittest

import mozunit
from manifestparser import ParseError, TestManifest
from manifestparser.filters import subsuite

here = os.path.dirname(os.path.abspath(__file__))


class TestTestManifest(unittest.TestCase):
    """Test the Test Manifest"""

    def test_testmanifest_toml(self):
        # Test filtering based on platform:
        filter_example = os.path.join(here, "filter-example.toml")
        manifest = TestManifest(
            manifests=(filter_example,), strict=False, use_toml=True
        )
        self.assertEqual(
            [
                i["name"]
                for i in manifest.active_tests(os="win", disabled=False, exists=False)
            ],
            ["windowstest", "fleem"],
        )
        self.assertEqual(
            [
                i["name"]
                for i in manifest.active_tests(os="linux", disabled=False, exists=False)
            ],
            ["fleem", "linuxtest"],
        )

        # Look for existing tests.  There is only one:
        self.assertEqual([i["name"] for i in manifest.active_tests()], ["fleem"])

        # You should be able to expect failures:
        last = manifest.active_tests(exists=False, os="linux")[-1]
        self.assertEqual(last["name"], "linuxtest")
        self.assertEqual(last["expected"], "pass")
        last = manifest.active_tests(exists=False, os="mac")[-1]
        self.assertEqual(last["expected"], "fail")

    def test_missing_paths_toml(self):
        """
        Test paths that don't exist raise an exception in strict mode. (TOML)
        """
        tempdir = tempfile.mkdtemp()

        missing_path = os.path.join(here, "missing-path.toml")
        manifest = TestManifest(manifests=(missing_path,), strict=True, use_toml=True)
        self.assertRaises(IOError, manifest.active_tests)
        self.assertRaises(IOError, manifest.copy, tempdir)
        self.assertRaises(IOError, manifest.update, tempdir)

        shutil.rmtree(tempdir)

    def test_comments_toml(self):
        """
        ensure comments work, see
        https://bugzilla.mozilla.org/show_bug.cgi?id=813674
        (TOML)
        """
        comment_example = os.path.join(here, "comment-example.toml")
        manifest = TestManifest(manifests=(comment_example,), use_toml=True)
        self.assertEqual(len(manifest.tests), 8)
        names = [i["name"] for i in manifest.tests]
        self.assertFalse("test_0202_app_launch_apply_update_dirlocked.js" in names)

    def test_manifest_subsuites_toml(self):
        """
        test subsuites and conditional subsuites (TOML)
        """
        relative_path = os.path.join(here, "subsuite.toml")
        manifest = TestManifest(manifests=(relative_path,), use_toml=True)
        info = {"foo": "bar"}

        # 6 tests total
        tests = manifest.active_tests(exists=False, **info)
        self.assertEqual(len(tests), 6)

        # only 3 tests for subsuite bar when foo==bar
        tests = manifest.active_tests(exists=False, filters=[subsuite("bar")], **info)
        self.assertEqual(len(tests), 3)

        # only 1 test for subsuite baz, regardless of conditions
        other = {"something": "else"}
        tests = manifest.active_tests(exists=False, filters=[subsuite("baz")], **info)
        self.assertEqual(len(tests), 1)
        tests = manifest.active_tests(exists=False, filters=[subsuite("baz")], **other)
        self.assertEqual(len(tests), 1)

        # 4 tests match when the condition doesn't match (all tests except
        # the unconditional subsuite)
        info = {"foo": "blah"}
        tests = manifest.active_tests(exists=False, filters=[subsuite()], **info)
        self.assertEqual(len(tests), 5)

        # test for illegal subsuite value
        manifest.tests[0]["subsuite"] = 'subsuite=bar,foo=="bar",type="nothing"'
        with self.assertRaises(ParseError):
            manifest.active_tests(exists=False, filters=[subsuite("foo")], **info)

    def test_none_and_empty_manifest_toml(self):
        """
        Test TestManifest for None and empty manifest, see
        https://bugzilla.mozilla.org/show_bug.cgi?id=1087682
        (TOML)
        """
        none_manifest = TestManifest(manifests=None, strict=False, use_toml=True)
        self.assertEqual(len(none_manifest.test_paths()), 0)
        self.assertEqual(len(none_manifest.active_tests()), 0)

        empty_manifest = TestManifest(manifests=[], strict=False)
        self.assertEqual(len(empty_manifest.test_paths()), 0)
        self.assertEqual(len(empty_manifest.active_tests()), 0)


if __name__ == "__main__":
    mozunit.main()



#!/usr/bin/env python

from time import sleep

import mozunit


def test_run_interactive(runner, create_thread):
    """Bug 965183: Run process in interactive mode and call wait()"""
    runner.start(interactive=True)

    thread = create_thread(runner, timeout=2)
    thread.start()

    # This is a blocking call. So the process should be killed by the thread
    runner.wait()
    thread.join()
    assert not runner.is_running()


def test_stop_interactive(runner):
    """Bug 965183: Explicitely stop process in interactive mode"""
    runner.start(interactive=True)
    runner.stop()


def test_wait_after_process_finished(runner):
    """Wait after the process has been stopped should not raise an error"""
    runner.start(interactive=True)
    sleep(1)
    runner.process_handler.kill()

    returncode = runner.wait(1)

    assert returncode not in [None, 0]
    assert runner.process_handler is not None


if __name__ == "__main__":
    mozunit.main()



#!/usr/bin/env python

import os
import tarfile
import tempfile
import zipfile

import mozfile
import mozunit
import pytest
import stubs


@pytest.fixture
def ensure_directory_contents():
    """ensure the directory contents match"""

    def inner(directory):
        for f in stubs.files:
            path = os.path.join(directory, *f)
            exists = os.path.exists(path)
            if not exists:
                print("%s does not exist" % (os.path.join(f)))
            assert exists
            if exists:
                contents = open(path).read().strip()
                assert contents == f[-1]

    return inner


@pytest.fixture(scope="module")
def tarpath(tmpdir_factory):
    """create a stub tarball for testing"""
    tmpdir = tmpdir_factory.mktemp("test_extract")

    tempdir = tmpdir.join("stubs").strpath
    stubs.create_stub(tempdir)
    filename = tmpdir.join("bundle.tar").strpath
    archive = tarfile.TarFile(filename, mode="w")
    for path in stubs.files:
        archive.add(os.path.join(tempdir, *path), arcname=os.path.join(*path))
    archive.close()

    assert os.path.exists(filename)
    return filename


@pytest.fixture(scope="module")
def zippath(tmpdir_factory):
    """create a stub zipfile for testing"""
    tmpdir = tmpdir_factory.mktemp("test_extract")

    tempdir = tmpdir.join("stubs").strpath
    stubs.create_stub(tempdir)
    filename = tmpdir.join("bundle.zip").strpath
    archive = zipfile.ZipFile(filename, mode="w")
    for path in stubs.files:
        archive.write(os.path.join(tempdir, *path), arcname=os.path.join(*path))
    archive.close()

    assert os.path.exists(filename)
    return filename


@pytest.fixture(scope="module", params=["tar", "zip"])
def bundlepath(request, tarpath, zippath):
    if request.param == "tar":
        return tarpath
    else:
        return zippath


def test_extract(tmpdir, bundlepath, ensure_directory_contents):
    """test extracting a zipfile"""
    dest = tmpdir.mkdir("dest").strpath
    mozfile.extract(bundlepath, dest)
    ensure_directory_contents(dest)


def test_extract_zipfile_missing_file_attributes(tmpdir):
    """if files do not have attributes set the default permissions have to be inherited."""
    _zipfile = os.path.join(
        os.path.dirname(__file__), "files", "missing_file_attributes.zip"
    )
    assert os.path.exists(_zipfile)
    dest = tmpdir.mkdir("dest").strpath

    # Get the default file permissions for the user
    fname = os.path.join(dest, "foo")
    with open(fname, "w"):
        pass
    default_stmode = os.stat(fname).st_mode

    files = mozfile.extract_zip(_zipfile, dest)
    for filename in files:
        assert os.stat(os.path.join(dest, filename)).st_mode == default_stmode


def test_extract_non_archive(tarpath, zippath):
    """test the generalized extract function"""
    # test extracting some non-archive; this should fail
    fd, filename = tempfile.mkstemp()
    os.write(fd, b"This is not a zipfile or tarball")
    os.close(fd)
    exception = None

    try:
        dest = tempfile.mkdtemp()
        mozfile.extract(filename, dest)
    except Exception as exc:
        exception = exc
    finally:
        os.remove(filename)
        os.rmdir(dest)

    assert isinstance(exception, Exception)


def test_extract_ignore(tmpdir, bundlepath):
    dest = tmpdir.mkdir("dest").strpath
    ignore = ("foo", "**/fleem.txt", "read*.txt")
    mozfile.extract(bundlepath, dest, ignore=ignore)

    assert sorted(os.listdir(dest)) == ["bar.txt", "foo.txt"]


def test_tarball_escape(tmpdir):
    """Ensures that extracting a tarball can't write outside of the intended
    destination directory.
    """
    workdir = tmpdir.mkdir("workdir")
    os.chdir(workdir)

    # Generate a "malicious" bundle.
    with open("bad.txt", "w") as fh:
        fh.write("pwned!")

    def change_name(tarinfo):
        tarinfo.name = "../" + tarinfo.name
        return tarinfo

    with tarfile.open("evil.tar", "w:xz") as tar:
        tar.add("bad.txt", filter=change_name)

    with pytest.raises(RuntimeError):
        mozfile.extract_tarball("evil.tar", workdir)
    assert not os.path.exists(tmpdir.join("bad.txt"))


if __name__ == "__main__":
    mozunit.main()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
import os
import shutil
import subprocess
import sys
from pathlib import Path
from textwrap import dedent

import mozunit
from buildconfig import topsrcdir

from mach.requirements import MachEnvRequirements
from mach.site import PythonVirtualenv


def _resolve_command_site_names():
    site_names = []
    for child in (Path(topsrcdir) / "python" / "sites").iterdir():
        if not child.is_file():
            continue

        if child.suffix != ".txt":
            continue

        if child.name == "mach.txt":
            continue

        site_names.append(child.stem)
    return site_names


def _requirement_definition_to_pip_format(site_name, cache, is_mach_or_build_env):
    """Convert from parsed requirements object to pip-consumable format"""
    requirements_path = Path(topsrcdir) / "python" / "sites" / f"{site_name}.txt"
    requirements = MachEnvRequirements.from_requirements_definition(
        topsrcdir, False, not is_mach_or_build_env, requirements_path
    )

    lines = []
    for pypi in (
        requirements.pypi_requirements + requirements.pypi_optional_requirements
    ):
        lines.append(str(pypi.requirement))

    for vendored in requirements.vendored_requirements:
        lines.append(str(cache.package_for_vendor_dir(Path(vendored.path))))

    for pth in requirements.pth_requirements:
        path = Path(pth.path)

        if "third_party" not in (p.name for p in path.parents):
            continue

        for child in path.iterdir():
            if child.name.endswith(".dist-info"):
                raise Exception(
                    f'In {requirements_path}, the "pth:" pointing to "{path}" has a '
                    '".dist-info" file.\n'
                    'Perhaps it should change to start with "vendored:" instead of '
                    '"pth:".'
                )
            if child.name.endswith(".egg-info"):
                raise Exception(
                    f'In {requirements_path}, the "pth:" pointing to "{path}" has an '
                    '".egg-info" file.\n'
                    'Perhaps it should change to start with "vendored:" instead of '
                    '"pth:".'
                )

    return "\n".join(lines)


class PackageCache:
    def __init__(self, storage_dir: Path):
        self._cache = {}
        self._storage_dir = storage_dir

    def package_for_vendor_dir(self, vendor_path: Path):
        if vendor_path in self._cache:
            return self._cache[vendor_path]

        if not any((p for p in vendor_path.iterdir() if p.name.endswith(".dist-info"))):
            # This vendored package is not a wheel. It may be a source package (with
            # a setup.py), or just some Python code that was manually copied into the
            # tree. If it's a source package, the setup.py file may be up a few levels
            # from the referenced Python module path.
            package_dir = vendor_path
            while True:
                if (package_dir / "setup.py").exists():
                    break
                elif package_dir.parent == package_dir:
                    raise Exception(
                        f'Package "{vendor_path}" is not a wheel and does not have a '
                        'setup.py file. Perhaps it should be "pth:" instead of '
                        '"vendored:"?'
                    )
                package_dir = package_dir.parent

            self._cache[vendor_path] = package_dir
            return package_dir

        # Pip requires that wheels have a version number in their name, even if
        # it ignores it. We should parse out the version and put it in here
        # so that failure debugging is easier, but that's non-trivial work.
        # So, this "0" satisfies pip's naming requirement while being relatively
        # obvious that it's a placeholder.
        output_path = self._storage_dir / f"{vendor_path.name}-0-py3-none-any"
        shutil.make_archive(str(output_path), "zip", vendor_path)

        whl_path = output_path.parent / (output_path.name + ".whl")
        (output_path.parent / (output_path.name + ".zip")).rename(whl_path)
        self._cache[vendor_path] = whl_path

        return whl_path


def test_sites_compatible(tmpdir: str):
    command_site_names = _resolve_command_site_names()
    work_dir = Path(tmpdir)
    cache = PackageCache(work_dir)
    mach_requirements = _requirement_definition_to_pip_format("mach", cache, True)

    # Create virtualenv to try to install all dependencies into.
    virtualenv = PythonVirtualenv(str(work_dir / "env"))
    subprocess.check_call(
        [
            sys.executable,
            "-m",
            "venv",
            "--without-pip",
            virtualenv.prefix,
        ]
    )
    platlib_dir = virtualenv.resolve_sysconfig_packages_path("platlib")
    third_party = Path(topsrcdir) / "third_party" / "python"
    with open(os.path.join(platlib_dir, "site.pth"), "w") as pthfile:
        pthfile.write(
            "\n".join(
                [
                    str(third_party / "pip"),
                    str(third_party / "wheel"),
                    str(third_party / "setuptools"),
                ]
            )
        )

    for name in command_site_names:
        print(f'Checking compatibility of "{name}" site')
        command_requirements = _requirement_definition_to_pip_format(
            name, cache, name == "build"
        )
        with open(work_dir / "requirements.txt", "w") as requirements_txt:
            requirements_txt.write(mach_requirements)
            requirements_txt.write("\n")
            requirements_txt.write(command_requirements)

        # Attempt to install combined set of dependencies (global Mach + current
        # command)
        proc = subprocess.run(
            [
                virtualenv.python_path,
                "-m",
                "pip",
                "install",
                "-r",
                str(work_dir / "requirements.txt"),
            ],
            cwd=topsrcdir,
        )
        if proc.returncode != 0:
            print(
                dedent(
                    f"""
                Error: The '{name}' site contains dependencies that are not
                compatible with the 'mach' site. Check the following files for
                any conflicting packages mentioned in the prior error message:

                  python/sites/mach.txt
                  python/sites/{name}.txt
                        """
                )
            )
            assert False


if __name__ == "__main__":
    mozunit.main()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this file,
# You can obtain one at http://mozilla.org/MPL/2.0/.

"""
VERSION CODES of the android releases.

See http://developer.android.com/reference/android/os/Build.VERSION_CODES.html.
"""
# Magic version number for a current development build, which has
# not yet turned into an official release.
CUR_DEVELOPMENT = 10000

# October 2008: The original, first, version of Android
BASE = 1
# February 2009: First Android update, officially called 1.1
BASE_1_1 = 2
# May 2009: Android 1.5
CUPCAKE = 3
# September 2009: Android 1.6
DONUT = 4
# November 2009: Android 2.0
ECLAIR = 5
# December 2009: Android 2.0.1
ECLAIR_0_1 = 6
# January 2010: Android 2.1
ECLAIR_MR1 = 7
# June 2010: Android 2.2
FROYO = 8
# November 2010: Android 2.3
GINGERBREAD = 9
# February 2011: Android 2.3.3
GINGERBREAD_MR1 = 10
# February 2011: Android 3.0
HONEYCOMB = 11
# May 2011: Android 3.1
HONEYCOMB_MR1 = 12
# June 2011: Android 3.2
HONEYCOMB_MR2 = 13
# October 2011: Android 4.0
ICE_CREAM_SANDWICH = 14
# December 2011: Android 4.0.3
ICE_CREAM_SANDWICH_MR1 = 15
# June 2012: Android 4.1
JELLY_BEAN = 16
# November 2012: Android 4.2
JELLY_BEAN_MR1 = 17
# July 2013: Android 4.3
JELLY_BEAN_MR2 = 18
# October 2013: Android 4.4
KITKAT = 19
# Android 4.4W
KITKAT_WATCH = 20
# Lollilop
LOLLIPOP = 21
LOLLIPOP_MR1 = 22
# Marshmallow
M = 23
# Nougat
N = 24
N_MR1 = 25
# Oreo
O = 26
O_MR1 = 27
# Pie
P = 28
# 10
Q = 29
# 11
R = 30



import eggsample


@eggsample.hookimpl
def eggsample_add_ingredients(ingredients):
    """Here the caller expects us to return a list."""
    if "egg" in ingredients:
        spam = ["lovely spam", "wonderous spam"]
    else:
        spam = ["splendiferous spam", "magnificent spam"]
    return spam


@eggsample.hookimpl
def eggsample_prep_condiments(condiments):
    """Here the caller passes a mutable object, so we mess with it directly."""
    try:
        del condiments["steak sauce"]
    except KeyError:
        pass
    condiments["spam sauce"] = 42
    return "Now this is what I call a condiments tray!"



from __future__ import absolute_import, print_function
import os.path
import shutil
import subprocess
import sys

VENDOR_TARGET = "py/_vendored_packages"
GOOD_FILES = ('README.md', '__init__.py')


def remove_libs():
    print("removing vendored libs")
    for filename in os.listdir(VENDOR_TARGET):
        if filename not in GOOD_FILES:
            path = os.path.join(VENDOR_TARGET, filename)
            print(" ", path)
            if os.path.isfile(path):
                os.remove(path)
            else:
                shutil.rmtree(path)


def update_libs():
    print("installing libs")
    subprocess.check_call((
        sys.executable, '-m', 'pip', 'install',
        '--target', VENDOR_TARGET, 'apipkg', 'iniconfig',
    ))
    subprocess.check_call(('git', 'add', VENDOR_TARGET))
    print("Please commit to finish the update after running the tests:")
    print()
    print('    git commit -am "Updated vendored libs"')


def main():
    remove_libs()
    update_libs()


if __name__ == '__main__':
    exit(main())



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import textwrap

from telemetry_harness.fog_ping_filters import FOG_DELETION_REQUEST_PING
from telemetry_harness.fog_testcase import FOGTestCase


class TestDeletionRequestPing(FOGTestCase):
    """Tests for FOG deletion-request ping."""

    def test_deletion_request_ping_across_sessions(self):
        """Test the "deletion-request" ping behaviour across sessions."""

        self.navigate_in_new_tab("about:glean")

        ping1 = self.wait_for_ping(
            self.disable_telemetry,
            FOG_DELETION_REQUEST_PING,
            ping_server=self.fog_ping_server,
        )

        self.assertIn("ping_info", ping1["payload"])
        self.assertIn("client_info", ping1["payload"])

        self.assertIn("client_id", ping1["payload"]["client_info"])
        client_id1 = ping1["payload"]["client_info"]["client_id"]
        self.assertIsValidUUID(client_id1)

        self.restart_browser()

        # We'd like to assert that a "deletion-request" is the last ping we
        # ever receive, but it's possible there's another ping on another
        # thread that gets sent after the sync-sent "deletion-request".
        # (This is fine, it'll be deleted within 28 days on the server.)
        # self.assertEqual(self.fog_ping_server.pings[-1], ping1)

        self.enable_telemetry()
        self.restart_browser()

        debug_tag = "my-test-tag"
        tagging_script = """\
        Services.fog.setTagPings("{}");
        """.format(
            debug_tag
        )
        with self.marionette.using_context(self.marionette.CONTEXT_CHROME):
            self.marionette.execute_script(textwrap.dedent(tagging_script))
        self.navigate_in_new_tab("about:glean")

        ping2 = self.wait_for_ping(
            self.disable_telemetry,
            FOG_DELETION_REQUEST_PING,
            ping_server=self.fog_ping_server,
        )

        self.assertEqual(ping2["debug_tag"].decode("utf-8"), debug_tag)

        self.assertIn("client_id", ping2["payload"]["client_info"])
        client_id2 = ping2["payload"]["client_info"]["client_id"]
        self.assertIsValidUUID(client_id2)

        # Verify that FOG creates a new client ID when a user
        # opts out of sending technical and interaction data.
        self.assertNotEqual(client_id2, client_id1)



# Copyright 2020, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""Standalone WebsocketServer

This file deals with the main module of standalone server. Although it is fine
to import this file directly to use WebSocketServer, it is strongly recommended
to use standalone.py, since it is intended to act as a skeleton of this module.
"""

from __future__ import absolute_import

import logging
import re
import select
import socket
import ssl
import threading
import traceback

from six.moves import BaseHTTPServer, socketserver

from pywebsocket3 import dispatch, util
from pywebsocket3.request_handler import WebSocketRequestHandler


def _alias_handlers(dispatcher, websock_handlers_map_file):
    """Set aliases specified in websock_handler_map_file in dispatcher.

    Args:
        dispatcher: dispatch.Dispatcher instance
        websock_handler_map_file: alias map file
    """

    with open(websock_handlers_map_file) as f:
        for line in f:
            if line[0] == '#' or line.isspace():
                continue
            m = re.match(r'(\S+)\s+(\S+)$', line)
            if not m:
                logging.warning('Wrong format in map file:' + line)
                continue
            try:
                dispatcher.add_resource_path_alias(m.group(1), m.group(2))
            except dispatch.DispatchException as e:
                logging.error(str(e))


class WebSocketServer(socketserver.ThreadingMixIn, BaseHTTPServer.HTTPServer):
    """HTTPServer specialized for WebSocket."""

    # Overrides SocketServer.ThreadingMixIn.daemon_threads
    daemon_threads = True
    # Overrides BaseHTTPServer.HTTPServer.allow_reuse_address
    allow_reuse_address = True

    def __init__(self, options):
        """Override SocketServer.TCPServer.__init__ to set SSL enabled
        socket object to self.socket before server_bind and server_activate,
        if necessary.
        """

        # Fall back to None for embedders that don't know about the
        # handler_encoding option.
        handler_encoding = getattr(options, "handler_encoding", None)

        # Share a Dispatcher among request handlers to save time for
        # instantiation.  Dispatcher can be shared because it is thread-safe.
        options.dispatcher = dispatch.Dispatcher(
            options.websock_handlers, options.scan_dir,
            options.allow_handlers_outside_root_dir, handler_encoding)
        if options.websock_handlers_map_file:
            _alias_handlers(options.dispatcher,
                            options.websock_handlers_map_file)
        warnings = options.dispatcher.source_warnings()
        if warnings:
            for warning in warnings:
                logging.warning('Warning in source loading: %s' % warning)

        self._logger = util.get_class_logger(self)

        self.request_queue_size = options.request_queue_size
        self.__ws_is_shut_down = threading.Event()
        self.__ws_serving = False

        socketserver.BaseServer.__init__(self,
                                         (options.server_host, options.port),
                                         WebSocketRequestHandler)

        # Expose the options object to allow handler objects access it. We name
        # it with websocket_ prefix to avoid conflict.
        self.websocket_server_options = options

        self._create_sockets()
        self.server_bind()
        self.server_activate()

    def _create_sockets(self):
        self.server_name, self.server_port = self.server_address
        self._sockets = []
        if not self.server_name:
            # On platforms that doesn't support IPv6, the first bind fails.
            # On platforms that supports IPv6
            # - If it binds both IPv4 and IPv6 on call with AF_INET6, the
            #   first bind succeeds and the second fails (we'll see 'Address
            #   already in use' error).
            # - If it binds only IPv6 on call with AF_INET6, both call are
            #   expected to succeed to listen both protocol.
            addrinfo_array = [(socket.AF_INET6, socket.SOCK_STREAM, '', '',
                               ''),
                              (socket.AF_INET, socket.SOCK_STREAM, '', '', '')]
        else:
            addrinfo_array = socket.getaddrinfo(self.server_name,
                                                self.server_port,
                                                socket.AF_UNSPEC,
                                                socket.SOCK_STREAM,
                                                socket.IPPROTO_TCP)
        for addrinfo in addrinfo_array:
            self._logger.info('Create socket on: %r', addrinfo)
            family, socktype, proto, canonname, sockaddr = addrinfo
            try:
                socket_ = socket.socket(family, socktype)
            except Exception as e:
                self._logger.info('Skip by failure: %r', e)
                continue
            server_options = self.websocket_server_options
            if server_options.use_tls:
                if server_options.tls_client_auth:
                    if server_options.tls_client_cert_optional:
                        client_cert_ = ssl.CERT_OPTIONAL
                    else:
                        client_cert_ = ssl.CERT_REQUIRED
                else:
                    client_cert_ = ssl.CERT_NONE
                ssl_context = ssl.SSLContext(ssl.PROTOCOL_TLS_SERVER)
                ssl_context.verify_mode = client_cert_
                ssl_context.load_cert_chain(keyfile=server_options.private_key,
                                            certfile=server_options.certificate)
                if client_cert_ != ssl.CERT_NONE:
                    ssl_context.load_verify_locations(cafile=server_options.tls_client_ca)
                socket_ = ssl_context.wrap_socket(socket_, server_side=True)
            self._sockets.append((socket_, addrinfo))

    def server_bind(self):
        """Override SocketServer.TCPServer.server_bind to enable multiple
        sockets bind.
        """

        failed_sockets = []

        for socketinfo in self._sockets:
            socket_, addrinfo = socketinfo
            self._logger.info('Bind on: %r', addrinfo)
            if self.allow_reuse_address:
                socket_.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            try:
                socket_.bind(self.server_address)
            except Exception as e:
                self._logger.info('Skip by failure: %r', e)
                socket_.close()
                failed_sockets.append(socketinfo)
            if self.server_address[1] == 0:
                # The operating system assigns the actual port number for port
                # number 0. This case, the second and later sockets should use
                # the same port number. Also self.server_port is rewritten
                # because it is exported, and will be used by external code.
                self.server_address = (self.server_name,
                                       socket_.getsockname()[1])
                self.server_port = self.server_address[1]
                self._logger.info('Port %r is assigned', self.server_port)

        for socketinfo in failed_sockets:
            self._sockets.remove(socketinfo)

    def server_activate(self):
        """Override SocketServer.TCPServer.server_activate to enable multiple
        sockets listen.
        """

        failed_sockets = []

        for socketinfo in self._sockets:
            socket_, addrinfo = socketinfo
            self._logger.info('Listen on: %r', addrinfo)
            try:
                socket_.listen(self.request_queue_size)
            except Exception as e:
                self._logger.info('Skip by failure: %r', e)
                socket_.close()
                failed_sockets.append(socketinfo)

        for socketinfo in failed_sockets:
            self._sockets.remove(socketinfo)

        if len(self._sockets) == 0:
            self._logger.critical(
                'No sockets activated. Use info log level to see the reason.')

    def server_close(self):
        """Override SocketServer.TCPServer.server_close to enable multiple
        sockets close.
        """

        for socketinfo in self._sockets:
            socket_, addrinfo = socketinfo
            self._logger.info('Close on: %r', addrinfo)
            socket_.close()

    def fileno(self):
        """Override SocketServer.TCPServer.fileno."""

        self._logger.critical('Not supported: fileno')
        return self._sockets[0][0].fileno()

    def handle_error(self, request, client_address):
        """Override SocketServer.handle_error."""

        self._logger.error('Exception in processing request from: %r\n%s',
                           client_address, traceback.format_exc())
        # Note: client_address is a tuple.

    def get_request(self):
        """Override TCPServer.get_request."""

        accepted_socket, client_address = self.socket.accept()

        server_options = self.websocket_server_options
        if server_options.use_tls:
            # Print cipher in use. Handshake is done on accept.
            self._logger.debug('Cipher: %s', accepted_socket.cipher())
            self._logger.debug('Client cert: %r',
                               accepted_socket.getpeercert())

        return accepted_socket, client_address

    def serve_forever(self, poll_interval=0.5):
        """Override SocketServer.BaseServer.serve_forever."""

        self.__ws_serving = True
        self.__ws_is_shut_down.clear()
        handle_request = self.handle_request
        if hasattr(self, '_handle_request_noblock'):
            handle_request = self._handle_request_noblock
        else:
            self._logger.warning('Fallback to blocking request handler')
        try:
            while self.__ws_serving:
                r, w, e = select.select(
                    [socket_[0] for socket_ in self._sockets], [], [],
                    poll_interval)
                for socket_ in r:
                    self.socket = socket_
                    handle_request()
                self.socket = None
        finally:
            self.__ws_is_shut_down.set()

    def shutdown(self):
        """Override SocketServer.BaseServer.shutdown."""

        self.__ws_serving = False
        self.__ws_is_shut_down.wait()


# vi:sts=4 sw=4 et



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this file,
# You can obtain one at http://mozilla.org/MPL/2.0/.


class L10n(object):
    """An API which allows Marionette to handle localized content.

    The `localization`_ of UI elements in Gecko based applications is done via
    entities and properties. For static values entities are used, which are located
    in .dtd files. Whereby for dynamically updated content the values come from
    .property files. Both types of elements can be identifed via a unique id,
    and the translated content retrieved.

    For example::

        from marionette_driver.localization import L10n
        l10n = L10n(marionette)

        l10n.localize_entity(["chrome://branding/locale/brand.dtd"], "brandShortName")
        l10n.localize_property(["chrome://global/locale/findbar.properties"], "FastFind"))

    .. _localization: https://mzl.la/2eUMjyF
    """

    def __init__(self, marionette):
        self._marionette = marionette

    def localize_entity(self, dtd_urls, entity_id):
        """Retrieve the localized string for the specified entity id.

        :param dtd_urls: List of .dtd URLs which will be used to search for the entity.
        :param entity_id: ID of the entity to retrieve the localized string for.

        :returns: The localized string for the requested entity.
        :raises: :exc:`NoSuchElementException`
        """
        body = {"urls": dtd_urls, "id": entity_id}
        return self._marionette._send_message("L10n:LocalizeEntity", body, key="value")

    def localize_property(self, properties_urls, property_id):
        """Retrieve the localized string for the specified property id.

        :param properties_urls: List of .properties URLs which will be used to
                                search for the property.
        :param property_id: ID of the property to retrieve the localized string for.

        :returns: The localized string for the requested property.
        :raises: :exc:`NoSuchElementException`
        """
        body = {"urls": properties_urls, "id": property_id}
        return self._marionette._send_message(
            "L10n:LocalizeProperty", body, key="value"
        )



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette_harness import BaseMarionetteTestRunner

from telemetry_harness.testcase import TelemetryTestCase

SERVER_URL = "http://localhost:8000"


class TelemetryTestRunner(BaseMarionetteTestRunner):
    """TestRunner for the telemetry-tests-client suite."""

    def __init__(self, **kwargs):
        """Set test variables and preferences specific to Firefox client
        telemetry.
        """

        # Select the appropriate GeckoInstance
        kwargs["app"] = "fxdesktop"

        prefs = kwargs.pop("prefs", {})

        prefs["fission.autostart"] = True
        if kwargs["disable_fission"]:
            prefs["fission.autostart"] = False

        # Set Firefox Client Telemetry specific preferences
        prefs.update(
            {
                # Clear the region detection url to
                #   * avoid net access in tests
                #   * stabilize browser.search.region to avoid extra subsessions (bug 1579840#c40)
                "browser.region.network.url": "",
                # Disable smart sizing because it changes prefs at startup. (bug 1547750)
                "browser.cache.disk.smart_size.enabled": False,
                "toolkit.telemetry.server": "{}/pings".format(SERVER_URL),
                "telemetry.fog.test.localhost_port": -1,
                "toolkit.telemetry.initDelay": 1,
                "toolkit.telemetry.minSubsessionLength": 0,
                "datareporting.healthreport.uploadEnabled": True,
                "datareporting.policy.dataSubmissionEnabled": True,
                "datareporting.policy.dataSubmissionPolicyBypassNotification": True,
                "toolkit.telemetry.log.level": "Trace",
                "toolkit.telemetry.log.dump": True,
                "toolkit.telemetry.send.overrideOfficialCheck": True,
                "toolkit.telemetry.testing.disableFuzzingDelay": True,
                # Disable Normandy to avoid extra subsessions due to Experiment
                # activation in tests (bug 1641571)
                "app.normandy.enabled": False,
                # Disable Normandy a little harder (bug 1608807).
                # This should also disable Nimbus.
                "app.shield.optoutstudies.enabled": False,
                # Bug 1789727: Keep the screenshots extension disabled to avoid
                # disabling the addon resulting in extra subsessions
                "screenshots.browser.component.enabled": False,
            }
        )

        super(TelemetryTestRunner, self).__init__(prefs=prefs, **kwargs)

        self.testvars["server_root"] = kwargs["server_root"]
        self.testvars["server_url"] = SERVER_URL

        self.test_handlers = [TelemetryTestCase]



#!/usr/bin/env python

import asyncio
import json

import aioredis
import django
import websockets

django.setup()

from django.contrib.contenttypes.models import ContentType
from sesame.utils import get_user
from websockets.frames import CloseCode


CONNECTIONS = {}


def get_content_types(user):
    """Return the set of IDs of content types visible by user."""
    # This does only three database queries because Django caches
    # all permissions on the first call to user.has_perm(...).
    return {
        ct.id
        for ct in ContentType.objects.all()
        if user.has_perm(f"{ct.app_label}.view_{ct.model}")
        or user.has_perm(f"{ct.app_label}.change_{ct.model}")
    }


async def handler(websocket):
    """Authenticate user and register connection in CONNECTIONS."""
    sesame = await websocket.recv()
    user = await asyncio.to_thread(get_user, sesame)
    if user is None:
        await websocket.close(CloseCode.INTERNAL_ERROR, "authentication failed")
        return

    ct_ids = await asyncio.to_thread(get_content_types, user)
    CONNECTIONS[websocket] = {"content_type_ids": ct_ids}
    try:
        await websocket.wait_closed()
    finally:
        del CONNECTIONS[websocket]


async def process_events():
    """Listen to events in Redis and process them."""
    redis = aioredis.from_url("redis://127.0.0.1:6379/1")
    pubsub = redis.pubsub()
    await pubsub.subscribe("events")
    async for message in pubsub.listen():
        if message["type"] != "message":
            continue
        payload = message["data"].decode()
        # Broadcast event to all users who have permissions to see it.
        event = json.loads(payload)
        recipients = (
            websocket
            for websocket, connection in CONNECTIONS.items()
            if event["content_type_id"] in connection["content_type_ids"]
        )
        websockets.broadcast(recipients, payload)


async def main():
    async with websockets.serve(handler, "localhost", 8888):
        await process_events()  # runs forever


if __name__ == "__main__":
    asyncio.run(main())



import re
import warnings
from typing import Optional

import pytest
from _pytest.pytester import Pytester
from _pytest.recwarn import WarningsRecorder


def test_recwarn_stacklevel(recwarn: WarningsRecorder) -> None:
    warnings.warn("hello")
    warn = recwarn.pop()
    assert warn.filename == __file__


def test_recwarn_functional(pytester: Pytester) -> None:
    pytester.makepyfile(
        """
        import warnings
        def test_method(recwarn):
            warnings.warn("hello")
            warn = recwarn.pop()
            assert isinstance(warn.message, UserWarning)
    """
    )
    reprec = pytester.inline_run()
    reprec.assertoutcome(passed=1)


@pytest.mark.filterwarnings("")
def test_recwarn_captures_deprecation_warning(recwarn: WarningsRecorder) -> None:
    """
    Check that recwarn can capture DeprecationWarning by default
    without custom filterwarnings (see #8666).
    """
    warnings.warn(DeprecationWarning("some deprecation"))
    assert len(recwarn) == 1
    assert recwarn.pop(DeprecationWarning)


class TestWarningsRecorderChecker:
    def test_recording(self) -> None:
        rec = WarningsRecorder(_ispytest=True)
        with rec:
            assert not rec.list
            warnings.warn_explicit("hello", UserWarning, "xyz", 13)
            assert len(rec.list) == 1
            warnings.warn(DeprecationWarning("hello"))
            assert len(rec.list) == 2
            warn = rec.pop()
            assert str(warn.message) == "hello"
            values = rec.list
            rec.clear()
            assert len(rec.list) == 0
            assert values is rec.list
            pytest.raises(AssertionError, rec.pop)

    def test_warn_stacklevel(self) -> None:
        """#4243"""
        rec = WarningsRecorder(_ispytest=True)
        with rec:
            warnings.warn("test", DeprecationWarning, 2)

    def test_typechecking(self) -> None:
        from _pytest.recwarn import WarningsChecker

        with pytest.raises(TypeError):
            WarningsChecker(5, _ispytest=True)  # type: ignore[arg-type]
        with pytest.raises(TypeError):
            WarningsChecker(("hi", RuntimeWarning), _ispytest=True)  # type: ignore[arg-type]
        with pytest.raises(TypeError):
            WarningsChecker([DeprecationWarning, RuntimeWarning], _ispytest=True)  # type: ignore[arg-type]

    def test_invalid_enter_exit(self) -> None:
        # wrap this test in WarningsRecorder to ensure warning state gets reset
        with WarningsRecorder(_ispytest=True):
            with pytest.raises(RuntimeError):
                rec = WarningsRecorder(_ispytest=True)
                rec.__exit__(None, None, None)  # can't exit before entering

            with pytest.raises(RuntimeError):
                rec = WarningsRecorder(_ispytest=True)
                with rec:
                    with rec:
                        pass  # can't enter twice


class TestDeprecatedCall:
    """test pytest.deprecated_call()"""

    def dep(self, i: int, j: Optional[int] = None) -> int:
        if i == 0:
            warnings.warn("is deprecated", DeprecationWarning, stacklevel=1)
        return 42

    def dep_explicit(self, i: int) -> None:
        if i == 0:
            warnings.warn_explicit(
                "dep_explicit", category=DeprecationWarning, filename="hello", lineno=3
            )

    def test_deprecated_call_raises(self) -> None:
        with pytest.raises(pytest.fail.Exception, match="No warnings of type"):
            pytest.deprecated_call(self.dep, 3, 5)

    def test_deprecated_call(self) -> None:
        pytest.deprecated_call(self.dep, 0, 5)

    def test_deprecated_call_ret(self) -> None:
        ret = pytest.deprecated_call(self.dep, 0)
        assert ret == 42

    def test_deprecated_call_preserves(self) -> None:
        # Type ignored because `onceregistry` and `filters` are not
        # documented API.
        onceregistry = warnings.onceregistry.copy()  # type: ignore
        filters = warnings.filters[:]  # type: ignore
        warn = warnings.warn
        warn_explicit = warnings.warn_explicit
        self.test_deprecated_call_raises()
        self.test_deprecated_call()
        assert onceregistry == warnings.onceregistry  # type: ignore
        assert filters == warnings.filters  # type: ignore
        assert warn is warnings.warn
        assert warn_explicit is warnings.warn_explicit

    def test_deprecated_explicit_call_raises(self) -> None:
        with pytest.raises(pytest.fail.Exception):
            pytest.deprecated_call(self.dep_explicit, 3)

    def test_deprecated_explicit_call(self) -> None:
        pytest.deprecated_call(self.dep_explicit, 0)
        pytest.deprecated_call(self.dep_explicit, 0)

    @pytest.mark.parametrize("mode", ["context_manager", "call"])
    def test_deprecated_call_no_warning(self, mode) -> None:
        """Ensure deprecated_call() raises the expected failure when its block/function does
        not raise a deprecation warning.
        """

        def f():
            pass

        msg = "No warnings of type (.*DeprecationWarning.*, .*PendingDeprecationWarning.*)"
        with pytest.raises(pytest.fail.Exception, match=msg):
            if mode == "call":
                pytest.deprecated_call(f)
            else:
                with pytest.deprecated_call():
                    f()

    @pytest.mark.parametrize(
        "warning_type", [PendingDeprecationWarning, DeprecationWarning]
    )
    @pytest.mark.parametrize("mode", ["context_manager", "call"])
    @pytest.mark.parametrize("call_f_first", [True, False])
    @pytest.mark.filterwarnings("ignore")
    def test_deprecated_call_modes(self, warning_type, mode, call_f_first) -> None:
        """Ensure deprecated_call() captures a deprecation warning as expected inside its
        block/function.
        """

        def f():
            warnings.warn(warning_type("hi"))
            return 10

        # ensure deprecated_call() can capture the warning even if it has already been triggered
        if call_f_first:
            assert f() == 10
        if mode == "call":
            assert pytest.deprecated_call(f) == 10
        else:
            with pytest.deprecated_call():
                assert f() == 10

    @pytest.mark.parametrize("mode", ["context_manager", "call"])
    def test_deprecated_call_exception_is_raised(self, mode) -> None:
        """If the block of the code being tested by deprecated_call() raises an exception,
        it must raise the exception undisturbed.
        """

        def f():
            raise ValueError("some exception")

        with pytest.raises(ValueError, match="some exception"):
            if mode == "call":
                pytest.deprecated_call(f)
            else:
                with pytest.deprecated_call():
                    f()

    def test_deprecated_call_specificity(self) -> None:
        other_warnings = [
            Warning,
            UserWarning,
            SyntaxWarning,
            RuntimeWarning,
            FutureWarning,
            ImportWarning,
            UnicodeWarning,
        ]
        for warning in other_warnings:

            def f():
                warnings.warn(warning("hi"))

            with pytest.raises(pytest.fail.Exception):
                pytest.deprecated_call(f)
            with pytest.raises(pytest.fail.Exception):
                with pytest.deprecated_call():
                    f()

    def test_deprecated_call_supports_match(self) -> None:
        with pytest.deprecated_call(match=r"must be \d+$"):
            warnings.warn("value must be 42", DeprecationWarning)

        with pytest.raises(pytest.fail.Exception):
            with pytest.deprecated_call(match=r"must be \d+$"):
                warnings.warn("this is not here", DeprecationWarning)


class TestWarns:
    def test_check_callable(self) -> None:
        source = "warnings.warn('w1', RuntimeWarning)"
        with pytest.raises(TypeError, match=r".* must be callable"):
            pytest.warns(RuntimeWarning, source)  # type: ignore

    def test_several_messages(self) -> None:
        # different messages, b/c Python suppresses multiple identical warnings
        pytest.warns(RuntimeWarning, lambda: warnings.warn("w1", RuntimeWarning))
        with pytest.raises(pytest.fail.Exception):
            pytest.warns(UserWarning, lambda: warnings.warn("w2", RuntimeWarning))
        pytest.warns(RuntimeWarning, lambda: warnings.warn("w3", RuntimeWarning))

    def test_function(self) -> None:
        pytest.warns(
            SyntaxWarning, lambda msg: warnings.warn(msg, SyntaxWarning), "syntax"
        )

    def test_warning_tuple(self) -> None:
        pytest.warns(
            (RuntimeWarning, SyntaxWarning), lambda: warnings.warn("w1", RuntimeWarning)
        )
        pytest.warns(
            (RuntimeWarning, SyntaxWarning), lambda: warnings.warn("w2", SyntaxWarning)
        )
        pytest.raises(
            pytest.fail.Exception,
            lambda: pytest.warns(
                (RuntimeWarning, SyntaxWarning),
                lambda: warnings.warn("w3", UserWarning),
            ),
        )

    def test_as_contextmanager(self) -> None:
        with pytest.warns(RuntimeWarning):
            warnings.warn("runtime", RuntimeWarning)

        with pytest.warns(UserWarning):
            warnings.warn("user", UserWarning)

        with pytest.raises(pytest.fail.Exception) as excinfo:
            with pytest.warns(RuntimeWarning):
                warnings.warn("user", UserWarning)
        excinfo.match(
            r"DID NOT WARN. No warnings of type \(.+RuntimeWarning.+,\) were emitted. "
            r"The list of emitted warnings is: \[UserWarning\('user',?\)\]."
        )

        with pytest.raises(pytest.fail.Exception) as excinfo:
            with pytest.warns(UserWarning):
                warnings.warn("runtime", RuntimeWarning)
        excinfo.match(
            r"DID NOT WARN. No warnings of type \(.+UserWarning.+,\) were emitted. "
            r"The list of emitted warnings is: \[RuntimeWarning\('runtime',?\)\]."
        )

        with pytest.raises(pytest.fail.Exception) as excinfo:
            with pytest.warns(UserWarning):
                pass
        excinfo.match(
            r"DID NOT WARN. No warnings of type \(.+UserWarning.+,\) were emitted. "
            r"The list of emitted warnings is: \[\]."
        )

        warning_classes = (UserWarning, FutureWarning)
        with pytest.raises(pytest.fail.Exception) as excinfo:
            with pytest.warns(warning_classes) as warninfo:
                warnings.warn("runtime", RuntimeWarning)
                warnings.warn("import", ImportWarning)

        message_template = (
            "DID NOT WARN. No warnings of type {0} were emitted. "
            "The list of emitted warnings is: {1}."
        )
        excinfo.match(
            re.escape(
                message_template.format(
                    warning_classes, [each.message for each in warninfo]
                )
            )
        )

    def test_record(self) -> None:
        with pytest.warns(UserWarning) as record:
            warnings.warn("user", UserWarning)

        assert len(record) == 1
        assert str(record[0].message) == "user"

    def test_record_only(self) -> None:
        with pytest.warns() as record:
            warnings.warn("user", UserWarning)
            warnings.warn("runtime", RuntimeWarning)

        assert len(record) == 2
        assert str(record[0].message) == "user"
        assert str(record[1].message) == "runtime"

    def test_record_only_none_deprecated_warn(self) -> None:
        # This should become an error when WARNS_NONE_ARG is removed in Pytest 8.0
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            with pytest.warns(None) as record:  # type: ignore[call-overload]
                warnings.warn("user", UserWarning)
                warnings.warn("runtime", RuntimeWarning)

            assert len(record) == 2
            assert str(record[0].message) == "user"
            assert str(record[1].message) == "runtime"

    def test_record_by_subclass(self) -> None:
        with pytest.warns(Warning) as record:
            warnings.warn("user", UserWarning)
            warnings.warn("runtime", RuntimeWarning)

        assert len(record) == 2
        assert str(record[0].message) == "user"
        assert str(record[1].message) == "runtime"

        class MyUserWarning(UserWarning):
            pass

        class MyRuntimeWarning(RuntimeWarning):
            pass

        with pytest.warns((UserWarning, RuntimeWarning)) as record:
            warnings.warn("user", MyUserWarning)
            warnings.warn("runtime", MyRuntimeWarning)

        assert len(record) == 2
        assert str(record[0].message) == "user"
        assert str(record[1].message) == "runtime"

    def test_double_test(self, pytester: Pytester) -> None:
        """If a test is run again, the warning should still be raised"""
        pytester.makepyfile(
            """
            import pytest
            import warnings

            @pytest.mark.parametrize('run', [1, 2])
            def test(run):
                with pytest.warns(RuntimeWarning):
                    warnings.warn("runtime", RuntimeWarning)
        """
        )
        result = pytester.runpytest()
        result.stdout.fnmatch_lines(["*2 passed in*"])

    def test_match_regex(self) -> None:
        with pytest.warns(UserWarning, match=r"must be \d+$"):
            warnings.warn("value must be 42", UserWarning)

        with pytest.raises(pytest.fail.Exception):
            with pytest.warns(UserWarning, match=r"must be \d+$"):
                warnings.warn("this is not here", UserWarning)

        with pytest.raises(pytest.fail.Exception):
            with pytest.warns(FutureWarning, match=r"must be \d+$"):
                warnings.warn("value must be 42", UserWarning)

    def test_one_from_multiple_warns(self) -> None:
        with pytest.warns(UserWarning, match=r"aaa"):
            warnings.warn("cccccccccc", UserWarning)
            warnings.warn("bbbbbbbbbb", UserWarning)
            warnings.warn("aaaaaaaaaa", UserWarning)

    def test_none_of_multiple_warns(self) -> None:
        with pytest.raises(pytest.fail.Exception):
            with pytest.warns(UserWarning, match=r"aaa"):
                warnings.warn("bbbbbbbbbb", UserWarning)
                warnings.warn("cccccccccc", UserWarning)

    @pytest.mark.filterwarnings("ignore")
    def test_can_capture_previously_warned(self) -> None:
        def f() -> int:
            warnings.warn(UserWarning("ohai"))
            return 10

        assert f() == 10
        assert pytest.warns(UserWarning, f) == 10
        assert pytest.warns(UserWarning, f) == 10
        assert pytest.warns(UserWarning, f) != "10"  # type: ignore[comparison-overlap]

    def test_warns_context_manager_with_kwargs(self) -> None:
        with pytest.raises(TypeError) as excinfo:
            with pytest.warns(UserWarning, foo="bar"):  # type: ignore
                pass
        assert "Unexpected keyword arguments" in str(excinfo.value)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from six.moves.urllib.parse import quote

from marionette_driver import By, errors, Wait
from marionette_driver.keys import Keys

from marionette_harness import (
    MarionetteTestCase,
    WindowManagerMixin,
)


class TestSendkeysMenupopup(WindowManagerMixin, MarionetteTestCase):
    def setUp(self):
        super(TestSendkeysMenupopup, self).setUp()

        self.marionette.set_context("chrome")
        new_window = self.open_chrome_window(
            "chrome://remote/content/marionette/test_menupopup.xhtml"
        )
        self.marionette.switch_to_window(new_window)

        self.click_el = self.marionette.find_element(By.ID, "options-button")
        self.disabled_menuitem_el = self.marionette.find_element(
            By.ID, "option-disabled"
        )
        self.hidden_menuitem_el = self.marionette.find_element(By.ID, "option-hidden")
        self.menuitem_el = self.marionette.find_element(By.ID, "option-enabled")
        self.menupopup_el = self.marionette.find_element(By.ID, "options-menupopup")
        self.testwindow_el = self.marionette.find_element(By.ID, "test-window")

    def context_menu_state(self):
        return self.menupopup_el.get_property("state")

    def open_context_menu(self):
        def attempt_open_context_menu():
            self.assertEqual(self.context_menu_state(), "closed")
            self.click_el.click()
            Wait(self.marionette).until(
                lambda _: self.context_menu_state() == "open",
                message="Context menu did not open",
            )

        try:
            attempt_open_context_menu()
        except errors.TimeoutException:
            # If the first attempt timed out, try a second time.
            # On Linux, the test will intermittently fail if we click too
            # early on the button. Retrying fixes the issue. See Bug 1686769.
            attempt_open_context_menu()

    def wait_for_context_menu_closed(self):
        Wait(self.marionette).until(
            lambda _: self.context_menu_state() == "closed",
            message="Context menu did not close",
        )

    def tearDown(self):
        try:
            self.close_all_windows()
        finally:
            super(TestSendkeysMenupopup, self).tearDown()

    def test_sendkeys_menuitem(self):
        # Try closing the context menu by sending ESCAPE to a visible context menu item.
        self.open_context_menu()

        self.menuitem_el.send_keys(Keys.ESCAPE)
        self.wait_for_context_menu_closed()

    def test_sendkeys_menupopup(self):
        # Try closing the context menu by sending ESCAPE to the context menu.
        self.open_context_menu()

        self.menupopup_el.send_keys(Keys.ESCAPE)
        self.wait_for_context_menu_closed()

    def test_sendkeys_window(self):
        # Try closing the context menu by sending ESCAPE to the main window.
        self.open_context_menu()

        self.testwindow_el.send_keys(Keys.ESCAPE)
        self.wait_for_context_menu_closed()

    def test_sendkeys_closed_menu(self):
        # send_keys should throw for the menupopup if the contextmenu is closed.
        with self.assertRaises(errors.ElementNotInteractableException):
            self.menupopup_el.send_keys(Keys.ESCAPE)

        # send_keys should throw for the menuitem if the contextmenu is closed.
        with self.assertRaises(errors.ElementNotInteractableException):
            self.menuitem_el.send_keys(Keys.ESCAPE)

    def test_sendkeys_hidden_disabled_menuitem(self):
        self.open_context_menu()

        # send_keys should throw for a disabled menuitem in an opened contextmenu.
        with self.assertRaises(errors.ElementNotInteractableException):
            self.disabled_menuitem_el.send_keys(Keys.ESCAPE)

        # send_keys should throw for a hidden menuitem in an opened contextmenu.
        with self.assertRaises(errors.ElementNotInteractableException):
            self.hidden_menuitem_el.send_keys(Keys.ESCAPE)



import contextlib
import ssl
import threading

from websockets.sync.server import *

from ..utils import CERTIFICATE


SERVER_CONTEXT = ssl.SSLContext(ssl.PROTOCOL_TLS_SERVER)
SERVER_CONTEXT.load_cert_chain(CERTIFICATE)

# Work around https://github.com/openssl/openssl/issues/7967

# This bug causes connect() to hang in tests for the client. Including this
# workaround acknowledges that the issue could happen outside of the test suite.

# It shouldn't happen too often, or else OpenSSL 1.1.1 would be unusable. If it
# happens, we can look for a library-level fix, but it won't be easy.

SERVER_CONTEXT.num_tickets = 0


def crash(ws):
    raise RuntimeError


def do_nothing(ws):
    pass


def eval_shell(ws):
    for expr in ws:
        value = eval(expr)
        ws.send(str(value))


class EvalShellMixin:
    def assertEval(self, client, expr, value):
        client.send(expr)
        self.assertEqual(client.recv(), value)


@contextlib.contextmanager
def run_server(ws_handler=eval_shell, host="localhost", port=0, **kwargs):
    with serve(ws_handler, host, port, **kwargs) as server:
        thread = threading.Thread(target=server.serve_forever)
        thread.start()
        try:
            yield server
        finally:
            server.shutdown()
            thread.join()


@contextlib.contextmanager
def run_unix_server(path, ws_handler=eval_shell, **kwargs):
    with unix_serve(ws_handler, path, **kwargs) as server:
        thread = threading.Thread(target=server.serve_forever)
        thread.start()
        try:
            yield server
        finally:
            server.shutdown()
            thread.join()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import os
import re
import sys
from pathlib import Path

sys.path.append(os.fspath(Path(__file__).parents[3] / "quota/test/marionette"))

from quota_test_case import QuotaTestCase

CACHEAPI_PBM_PREF = "dom.cache.privateBrowsing.enabled"
QM_TESTING_PREF = "dom.quotaManager.testing"


class CacheAPIEncryptionPBM(QuotaTestCase):

    """
    Bug1856953: Ensure CacheAPI data gets encrypted in Private Browsing Mode.
    We need to ensure data inside both sqlite fields and request/response files
    gets encrypted
    """

    def setUp(self):
        super(CacheAPIEncryptionPBM, self).setUp()

        self.testHTML = "dom/cache/basicCacheAPI_PBM.html"
        self.cacheName = "CachePBMTest"
        self.profilePath = self.marionette.instance.profile.profile
        self.cacheAPIStoragePath = None

        self.defaultCacheAPIPBMPref = self.marionette.get_pref(CACHEAPI_PBM_PREF)
        self.marionette.set_pref(CACHEAPI_PBM_PREF, True)

        self.defaultQMPrefValue = self.marionette.get_pref(QM_TESTING_PREF)
        self.marionette.set_pref(QM_TESTING_PREF, True)

        self.cacheRequestStr = "https://example.com/"
        self.cacheResponseStr = "CacheAPIEncryptionPBM"

        self.cacheDBFileName = "caches.sqlite"
        self.cacheDBJournalFileName = "caches.sqlite-wal"

        self.dbCheckpointThresholdBytes = 512 * 1024

    def tearDown(self):
        super(CacheAPIEncryptionPBM, self).setUp()

        self.marionette.set_pref(CACHEAPI_PBM_PREF, self.defaultCacheAPIPBMPref)
        self.marionette.set_pref(QM_TESTING_PREF, self.defaultQMPrefValue)

    def test_request_response_ondisk(self):
        with self.using_new_window(self.testHTML, private=False) as (
            self.origin,
            self.persistenceType,
        ):
            self.runAndValidate(
                lambda exists: self.assertTrue(
                    exists, "Failed to find expected data on disk"
                )
            )

    def test_encrypted_request_response_ondisk(self):
        with self.using_new_window(self.testHTML, private=True) as (
            self.origin,
            self.persistenceType,
        ):
            self.runAndValidate(
                lambda exists: self.assertFalse(exists, "Data on disk is not encrypted")
            )

    def runAndValidate(self, validator):
        self.marionette.execute_async_script(
            """
                const [name, requestStr, responseStr, resolve] = arguments;

                const request = new Request(requestStr);
                const response = new Response(responseStr);
                window.wrappedJSObject.addDataIntoCache(name, request, response)
                                    .then(resolve);
            """,
            script_args=(
                self.cacheName,
                self.cacheRequestStr,
                self.cacheResponseStr,
            ),
        )

        self.ensureInvariantHolds(
            lambda _: os.path.exists(self.getCacheAPIStoragePath())
        )

        self.validateSqlite(validator)
        self.validateBodyFile(validator)

    def validateBodyFile(self, validator):
        # Ensure response bodies have been flushed to the disk
        self.ensureInvariantHolds(
            lambda _: self.findDirObj(self.getCacheAPIStoragePath(), "morgue", False)
            is not None
        )

        cacheResponseDir = self.findDirObj(
            self.getCacheAPIStoragePath(), "morgue", False
        )

        self.ensureInvariantHolds(lambda _: any(os.listdir(cacheResponseDir)))

        # Get response bodies directory corresponding to the cache 'self.CacheName' since, there's
        # only one cache object in this origin, it must be the first one.
        cacheResponseBodiesPath = [
            d for d in Path(cacheResponseDir).iterdir() if d.is_dir()
        ][0]

        # Ensure bodies have been transferred to '.final' from '.tmp'
        self.ensureInvariantHolds(
            lambda _: self.findDirObj(cacheResponseBodiesPath, ".final", True)
            is not None
        )
        cacheResponseBodyPath = self.findDirObj(cacheResponseBodiesPath, ".final", True)

        # Since a cache response would get compressed using snappy; and an unencrypted response would
        # contain 'sNaPpY' as a compression header in the response body file. Check to ensure that
        # 'sNaPpy' does not exist if bodies are getting encrypted.
        foundRawValue = False
        with open(cacheResponseBodyPath, "rb") as f_binary:
            foundRawValue = re.search(b"sNaPpY", f_binary.read()) is not None

        validator(foundRawValue)

    def validateSqlite(self, validator):
        self.ensureInvariantHolds(
            lambda _: self.findDirObj(
                self.getCacheAPIStoragePath(), self.cacheDBJournalFileName, True
            )
            is not None
        )
        dbJournalFile = self.findDirObj(
            self.getCacheAPIStoragePath(), self.cacheDBJournalFileName, True
        )

        self.ensureInvariantHolds(
            lambda _: self.findDirObj(
                self.getCacheAPIStoragePath(), self.cacheDBFileName, True
            )
            is not None
        )
        dbFile = self.findDirObj(
            self.getCacheAPIStoragePath(), self.cacheDBFileName, True
        )

        # Confirm journal file size is less than 512KB which ensures that checkpoint
        # has not happend yet (dom/cache/DBSchema.cpp::InitializeConnection, kWalAutoCheckpointPages)
        self.assertTrue(
            os.path.getsize(dbJournalFile) < self.dbCheckpointThresholdBytes
        )

        # Before checkpointing, journal file size should be greater than main sqlite db file.
        self.assertTrue(os.path.getsize(dbJournalFile) > os.path.getsize(dbFile))

        validator(
            self.cacheRequestStr.encode("ascii") in open(dbJournalFile, "rb").read()
        )

        self.assertTrue(
            self.resetStoragesForPrincipal(self.origin, self.persistenceType, "cache")
        )

        self.assertFalse(os.path.getsize(dbJournalFile) > os.path.getsize(dbFile))

        validator(self.cacheRequestStr.encode("ascii") in open(dbFile, "rb").read())

    def getCacheAPIStoragePath(self):
        if self.cacheAPIStoragePath is not None:
            return self.cacheAPIStoragePath

        assert self.origin is not None
        assert self.persistenceType is not None

        self.cacheAPIStoragePath = self.getStoragePath(
            self.profilePath, self.origin, self.persistenceType, "cache"
        )

        print("cacheAPI origin directory = " + self.cacheAPIStoragePath)
        return self.cacheAPIStoragePath



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import functools
import types
from unittest.case import SkipTest


def parameterized(func_suffix, *args, **kwargs):
    r"""Decorator which generates methods given a base method and some data.

    **func_suffix** is used as a suffix for the new created method and must be
    unique given a base method. if **func_suffix** countains characters that
    are not allowed in normal python function name, these characters will be
    replaced with "_".

    This decorator can be used more than once on a single base method. The class
    must have a metaclass of :class:`MetaParameterized`.

    Example::

      # This example will generate two methods:
      #
      # - MyTestCase.test_it_1
      # - MyTestCase.test_it_2
      #
      class MyTestCase(MarionetteTestCase):
          @parameterized("1", 5, named='name')
          @parameterized("2", 6, named='name2')
          def test_it(self, value, named=None):
              print value, named

    :param func_suffix: will be used as a suffix for the new method
    :param \*args: arguments to pass to the new method
    :param \*\*kwargs: named arguments to pass to the new method
    """

    def wrapped(func):
        if not hasattr(func, "metaparameters"):
            func.metaparameters = []
        func.metaparameters.append((func_suffix, args, kwargs))
        return func

    return wrapped


def run_if_manage_instance(reason):
    """Decorator which runs a test if Marionette manages the application instance."""

    def decorator(test_item):
        if not isinstance(test_item, types.FunctionType):
            raise Exception("Decorator only supported for functions")

        @functools.wraps(test_item)
        def skip_wrapper(self, *args, **kwargs):
            if self.marionette.instance is None:
                raise SkipTest(reason)
            return test_item(self, *args, **kwargs)

        return skip_wrapper

    return decorator


def skip_if_chrome(reason):
    """Decorator which skips a test if chrome context is active."""

    def decorator(test_item):
        if not isinstance(test_item, types.FunctionType):
            raise Exception("Decorator only supported for functions")

        @functools.wraps(test_item)
        def skip_wrapper(self, *args, **kwargs):
            if self.marionette._send_message("getContext", key="value") == "chrome":
                raise SkipTest(reason)
            return test_item(self, *args, **kwargs)

        return skip_wrapper

    return decorator


def skip_if_desktop(reason):
    """Decorator which skips a test if run on desktop."""

    def decorator(test_item):
        if not isinstance(test_item, types.FunctionType):
            raise Exception("Decorator only supported for functions")

        @functools.wraps(test_item)
        def skip_wrapper(self, *args, **kwargs):
            if self.marionette.session_capabilities.get("browserName") == "firefox":
                raise SkipTest(reason)
            return test_item(self, *args, **kwargs)

        return skip_wrapper

    return decorator


def skip_unless_browser_pref(reason, pref, predicate=bool):
    """Decorator which skips a test based on the value of a browser preference.

    :param reason: Message describing why the test need to be skipped.
    :param pref: the preference name
    :param predicate: a function that should return false to skip the test.
                      The function takes one parameter, the preference value.
                      Defaults to the python built-in bool function.

    Note that the preference must exist, else a failure is raised.

    Example: ::

      class TestSomething(MarionetteTestCase):
          @skip_unless_browser_pref("Sessionstore needs to be enabled for crashes",
                                    "browser.sessionstore.resume_from_crash",
                                    lambda value: value is True,
                                    )
          def test_foo(self):
              pass  # test implementation here

    """

    def decorator(test_item):
        if not isinstance(test_item, types.FunctionType):
            raise Exception("Decorator only supported for functions")
        if not callable(predicate):
            raise ValueError("predicate must be callable")

        @functools.wraps(test_item)
        def skip_wrapper(self, *args, **kwargs):
            value = self.marionette.get_pref(pref)
            if value is None:
                self.fail("No such browser preference: {0!r}".format(pref))
            if not predicate(value):
                raise SkipTest(reason)
            return test_item(self, *args, **kwargs)

        return skip_wrapper

    return decorator


def skip_unless_protocol(reason, predicate):
    """Decorator which skips a test if the predicate does not match the current protocol level."""

    def decorator(test_item):
        if not isinstance(test_item, types.FunctionType):
            raise Exception("Decorator only supported for functions")
        if not callable(predicate):
            raise ValueError("predicate must be callable")

        @functools.wraps(test_item)
        def skip_wrapper(self, *args, **kwargs):
            level = self.marionette.client.protocol
            if not predicate(level):
                raise SkipTest(reason)
            return test_item(self, *args, **kwargs)

        return skip_wrapper

    return decorator


def with_parameters(parameters):
    """Decorator which generates methods given a base method and some data.

    Acts like :func:`parameterized`, but define all methods in one call.

    Example::

      # This example will generate two methods:
      #
      # - MyTestCase.test_it_1
      # - MyTestCase.test_it_2
      #

      DATA = [("1", [5], {'named':'name'}), ("2", [6], {'named':'name2'})]

      class MyTestCase(MarionetteTestCase):
          @with_parameters(DATA)
          def test_it(self, value, named=None):
              print value, named

    :param parameters: list of tuples (**func_suffix**, **args**, **kwargs**)
                       defining parameters like in :func:`todo`.
    """

    def wrapped(func):
        func.metaparameters = parameters
        return func

    return wrapped



from setuptools import setup, find_packages
import io
from os import path
import re


VERSION = re.search("VERSION = '([^']+)'", io.open(
    path.join(path.dirname(__file__), 'webencodings', '__init__.py'),
    encoding='utf-8'
).read().strip()).group(1)

LONG_DESCRIPTION = io.open(
    path.join(path.dirname(__file__), 'README.rst'),
    encoding='utf-8'
).read()


setup(
    name='webencodings',
    version=VERSION,
    url='https://github.com/SimonSapin/python-webencodings',
    license='BSD',
    author='Simon Sapin',
    author_email='simon.sapin@exyr.org',
    maintainer='Geoffrey Sneddon',
    maintainer_email='me@gsnedders.com',
    description='Character encoding aliases for legacy web content',
    long_description=LONG_DESCRIPTION,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Internet :: WWW/HTTP',
    ],
    packages=find_packages(),
)



#!/usr/bin/env python
#
# Copyright 2011, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""Tests for handshake module."""

from __future__ import absolute_import

import unittest

import set_sys_path  # Update sys.path to locate pywebsocket3 module.
from pywebsocket3 import common
from pywebsocket3.handshake.base import (
    AbortedByUserException,
    HandshakeException,
    VersionException,
)
from pywebsocket3.handshake.hybi import Handshaker
from test import mock


class RequestDefinition(object):
    """A class for holding data for constructing opening handshake strings for
    testing the opening handshake processor.
    """
    def __init__(self, method, uri, headers):
        self.method = method
        self.uri = uri
        self.headers = headers


def _create_good_request_def():
    return RequestDefinition(
        'GET', '/demo', {
            'Host': 'server.example.com',
            'Upgrade': 'websocket',
            'Connection': 'Upgrade',
            'Sec-WebSocket-Key': 'dGhlIHNhbXBsZSBub25jZQ==',
            'Sec-WebSocket-Version': '13',
            'Origin': 'http://example.com'
        })


def _create_request(request_def):
    conn = mock.MockConn(b'')
    return mock.MockRequest(method=request_def.method,
                            uri=request_def.uri,
                            headers_in=request_def.headers,
                            connection=conn)


def _create_handshaker(request):
    handshaker = Handshaker(request, mock.MockDispatcher())
    return handshaker


class SubprotocolChoosingDispatcher(object):
    """A dispatcher for testing. This dispatcher sets the i-th subprotocol
    of requested ones to ws_protocol where i is given on construction as index
    argument. If index is negative, default_value will be set to ws_protocol.
    """
    def __init__(self, index, default_value=None):
        self.index = index
        self.default_value = default_value

    def do_extra_handshake(self, conn_context):
        if self.index >= 0:
            conn_context.ws_protocol = conn_context.ws_requested_protocols[
                self.index]
        else:
            conn_context.ws_protocol = self.default_value

    def transfer_data(self, conn_context):
        pass


class HandshakeAbortedException(Exception):
    pass


class AbortingDispatcher(object):
    """A dispatcher for testing. This dispatcher raises an exception in
    do_extra_handshake to reject the request.
    """
    def do_extra_handshake(self, conn_context):
        raise HandshakeAbortedException('An exception to reject the request')

    def transfer_data(self, conn_context):
        pass


class AbortedByUserDispatcher(object):
    """A dispatcher for testing. This dispatcher raises an
    AbortedByUserException in do_extra_handshake to reject the request.
    """
    def do_extra_handshake(self, conn_context):
        raise AbortedByUserException('An AbortedByUserException to reject the '
                                     'request')

    def transfer_data(self, conn_context):
        pass


_EXPECTED_RESPONSE = (
    b'HTTP/1.1 101 Switching Protocols\r\n'
    b'Upgrade: websocket\r\n'
    b'Connection: Upgrade\r\n'
    b'Sec-WebSocket-Accept: s3pPLMBiTxaQ9kYGzzhZRbK+xOo=\r\n\r\n')


class HandshakerTest(unittest.TestCase):
    """A unittest for draft-ietf-hybi-thewebsocketprotocol-06 and later
    handshake processor.
    """
    def test_do_handshake(self):
        request = _create_request(_create_good_request_def())
        dispatcher = mock.MockDispatcher()
        handshaker = Handshaker(request, dispatcher)
        handshaker.do_handshake()

        self.assertTrue(dispatcher.do_extra_handshake_called)

        self.assertEqual(_EXPECTED_RESPONSE, request.connection.written_data())
        self.assertEqual('/demo', request.ws_resource)
        self.assertEqual('http://example.com', request.ws_origin)
        self.assertEqual(None, request.ws_protocol)
        self.assertEqual(None, request.ws_extensions)
        self.assertEqual(common.VERSION_HYBI_LATEST, request.ws_version)

    def test_do_handshake_with_extra_headers(self):
        request_def = _create_good_request_def()
        # Add headers not related to WebSocket opening handshake.
        request_def.headers['FooKey'] = 'BarValue'
        request_def.headers['EmptyKey'] = ''

        request = _create_request(request_def)
        handshaker = _create_handshaker(request)
        handshaker.do_handshake()
        self.assertEqual(_EXPECTED_RESPONSE, request.connection.written_data())

    def test_do_handshake_with_capitalized_value(self):
        request_def = _create_good_request_def()
        request_def.headers['upgrade'] = 'WEBSOCKET'

        request = _create_request(request_def)
        handshaker = _create_handshaker(request)
        handshaker.do_handshake()
        self.assertEqual(_EXPECTED_RESPONSE, request.connection.written_data())

        request_def = _create_good_request_def()
        request_def.headers['Connection'] = 'UPGRADE'

        request = _create_request(request_def)
        handshaker = _create_handshaker(request)
        handshaker.do_handshake()
        self.assertEqual(_EXPECTED_RESPONSE, request.connection.written_data())

    def test_do_handshake_with_multiple_connection_values(self):
        request_def = _create_good_request_def()
        request_def.headers['Connection'] = 'Upgrade, keep-alive, , '

        request = _create_request(request_def)
        handshaker = _create_handshaker(request)
        handshaker.do_handshake()
        self.assertEqual(_EXPECTED_RESPONSE, request.connection.written_data())

    def test_aborting_handshake(self):
        handshaker = Handshaker(_create_request(_create_good_request_def()),
                                AbortingDispatcher())
        # do_extra_handshake raises an exception. Check that it's not caught by
        # do_handshake.
        self.assertRaises(HandshakeAbortedException, handshaker.do_handshake)

    def test_do_handshake_with_protocol(self):
        request_def = _create_good_request_def()
        request_def.headers['Sec-WebSocket-Protocol'] = 'chat, superchat'

        request = _create_request(request_def)
        handshaker = Handshaker(request, SubprotocolChoosingDispatcher(0))
        handshaker.do_handshake()

        EXPECTED_RESPONSE = (
            b'HTTP/1.1 101 Switching Protocols\r\n'
            b'Upgrade: websocket\r\n'
            b'Connection: Upgrade\r\n'
            b'Sec-WebSocket-Accept: s3pPLMBiTxaQ9kYGzzhZRbK+xOo=\r\n'
            b'Sec-WebSocket-Protocol: chat\r\n\r\n')

        self.assertEqual(EXPECTED_RESPONSE, request.connection.written_data())
        self.assertEqual('chat', request.ws_protocol)

    def test_do_handshake_protocol_not_in_request_but_in_response(self):
        request_def = _create_good_request_def()
        request = _create_request(request_def)
        handshaker = Handshaker(request,
                                SubprotocolChoosingDispatcher(-1, 'foobar'))
        # No request has been made but ws_protocol is set. HandshakeException
        # must be raised.
        self.assertRaises(HandshakeException, handshaker.do_handshake)

    def test_do_handshake_with_protocol_no_protocol_selection(self):
        request_def = _create_good_request_def()
        request_def.headers['Sec-WebSocket-Protocol'] = 'chat, superchat'

        request = _create_request(request_def)
        handshaker = _create_handshaker(request)
        # ws_protocol is not set. HandshakeException must be raised.
        self.assertRaises(HandshakeException, handshaker.do_handshake)

    def test_do_handshake_with_extensions(self):
        request_def = _create_good_request_def()
        request_def.headers['Sec-WebSocket-Extensions'] = (
            'permessage-deflate; server_no_context_takeover')

        EXPECTED_RESPONSE = (
            b'HTTP/1.1 101 Switching Protocols\r\n'
            b'Upgrade: websocket\r\n'
            b'Connection: Upgrade\r\n'
            b'Sec-WebSocket-Accept: s3pPLMBiTxaQ9kYGzzhZRbK+xOo=\r\n'
            b'Sec-WebSocket-Extensions: '
            b'permessage-deflate; server_no_context_takeover\r\n'
            b'\r\n')

        request = _create_request(request_def)
        handshaker = _create_handshaker(request)
        handshaker.do_handshake()
        self.assertEqual(EXPECTED_RESPONSE, request.connection.written_data())
        self.assertEqual(1, len(request.ws_extensions))
        extension = request.ws_extensions[0]
        self.assertEqual(common.PERMESSAGE_DEFLATE_EXTENSION, extension.name())
        self.assertEqual(['server_no_context_takeover'],
                         extension.get_parameter_names())
        self.assertEqual(
            None, extension.get_parameter_value('server_no_context_takeover'))
        self.assertEqual(1, len(request.ws_extension_processors))
        self.assertEqual('deflate', request.ws_extension_processors[0].name())

    def test_do_handshake_with_quoted_extensions(self):
        request_def = _create_good_request_def()
        request_def.headers['Sec-WebSocket-Extensions'] = (
            'permessage-deflate, , '
            'unknown; e   =    "mc^2"; ma="\r\n      \\\rf  "; pv=nrt')

        request = _create_request(request_def)
        handshaker = _create_handshaker(request)
        handshaker.do_handshake()
        self.assertEqual(2, len(request.ws_requested_extensions))
        first_extension = request.ws_requested_extensions[0]
        self.assertEqual('permessage-deflate', first_extension.name())
        second_extension = request.ws_requested_extensions[1]
        self.assertEqual('unknown', second_extension.name())
        self.assertEqual(['e', 'ma', 'pv'],
                         second_extension.get_parameter_names())
        self.assertEqual('mc^2', second_extension.get_parameter_value('e'))
        self.assertEqual(' \rf ', second_extension.get_parameter_value('ma'))
        self.assertEqual('nrt', second_extension.get_parameter_value('pv'))

    def test_do_handshake_with_optional_headers(self):
        request_def = _create_good_request_def()
        request_def.headers['EmptyValue'] = ''
        request_def.headers['AKey'] = 'AValue'

        request = _create_request(request_def)
        handshaker = _create_handshaker(request)
        handshaker.do_handshake()
        self.assertEqual('AValue', request.headers_in['AKey'])
        self.assertEqual('', request.headers_in['EmptyValue'])

    def test_abort_extra_handshake(self):
        handshaker = Handshaker(_create_request(_create_good_request_def()),
                                AbortedByUserDispatcher())
        # do_extra_handshake raises an AbortedByUserException. Check that it's
        # not caught by do_handshake.
        self.assertRaises(AbortedByUserException, handshaker.do_handshake)

    def test_bad_requests(self):
        bad_cases = [
            ('HTTP request',
             RequestDefinition(
                 'GET', '/demo', {
                     'Host':
                     'www.google.com',
                     'User-Agent':
                     'Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5;'
                     ' en-US; rv:1.9.1.3) Gecko/20090824 Firefox/3.5.3'
                     ' GTB6 GTBA',
                     'Accept':
                     'text/html,application/xhtml+xml,application/xml;q=0.9,'
                     '*/*;q=0.8',
                     'Accept-Language':
                     'en-us,en;q=0.5',
                     'Accept-Encoding':
                     'gzip,deflate',
                     'Accept-Charset':
                     'ISO-8859-1,utf-8;q=0.7,*;q=0.7',
                     'Keep-Alive':
                     '300',
                     'Connection':
                     'keep-alive'
                 }), None, True)
        ]

        request_def = _create_good_request_def()
        request_def.method = 'POST'
        bad_cases.append(('Wrong method', request_def, None, True))

        request_def = _create_good_request_def()
        del request_def.headers['Host']
        bad_cases.append(('Missing Host', request_def, None, True))

        request_def = _create_good_request_def()
        del request_def.headers['Upgrade']
        bad_cases.append(('Missing Upgrade', request_def, None, True))

        request_def = _create_good_request_def()
        request_def.headers['Upgrade'] = 'nonwebsocket'
        bad_cases.append(('Wrong Upgrade', request_def, None, True))

        request_def = _create_good_request_def()
        del request_def.headers['Connection']
        bad_cases.append(('Missing Connection', request_def, None, True))

        request_def = _create_good_request_def()
        request_def.headers['Connection'] = 'Downgrade'
        bad_cases.append(('Wrong Connection', request_def, None, True))

        request_def = _create_good_request_def()
        del request_def.headers['Sec-WebSocket-Key']
        bad_cases.append(('Missing Sec-WebSocket-Key', request_def, 400, True))

        request_def = _create_good_request_def()
        request_def.headers['Sec-WebSocket-Key'] = (
            'dGhlIHNhbXBsZSBub25jZQ==garbage')
        bad_cases.append(('Wrong Sec-WebSocket-Key (with garbage on the tail)',
                          request_def, 400, True))

        request_def = _create_good_request_def()
        request_def.headers['Sec-WebSocket-Key'] = 'YQ=='  # BASE64 of 'a'
        bad_cases.append(
            ('Wrong Sec-WebSocket-Key (decoded value is not 16 octets long)',
             request_def, 400, True))

        request_def = _create_good_request_def()
        # The last character right before == must be any of A, Q, w and g.
        request_def.headers['Sec-WebSocket-Key'] = 'AQIDBAUGBwgJCgsMDQ4PEC=='
        bad_cases.append(
            ('Wrong Sec-WebSocket-Key (padding bits are not zero)',
             request_def, 400, True))

        request_def = _create_good_request_def()
        request_def.headers['Sec-WebSocket-Key'] = (
            'dGhlIHNhbXBsZSBub25jZQ==,dGhlIHNhbXBsZSBub25jZQ==')
        bad_cases.append(('Wrong Sec-WebSocket-Key (multiple values)',
                          request_def, 400, True))

        request_def = _create_good_request_def()
        del request_def.headers['Sec-WebSocket-Version']
        bad_cases.append(
            ('Missing Sec-WebSocket-Version', request_def, None, True))

        request_def = _create_good_request_def()
        request_def.headers['Sec-WebSocket-Version'] = '3'
        bad_cases.append(
            ('Wrong Sec-WebSocket-Version', request_def, None, False))

        request_def = _create_good_request_def()
        request_def.headers['Sec-WebSocket-Version'] = '13, 13'
        bad_cases.append(('Wrong Sec-WebSocket-Version (multiple values)',
                          request_def, 400, True))

        request_def = _create_good_request_def()
        request_def.headers['Sec-WebSocket-Protocol'] = 'illegal\x09protocol'
        bad_cases.append(
            ('Illegal Sec-WebSocket-Protocol', request_def, 400, True))

        request_def = _create_good_request_def()
        request_def.headers['Sec-WebSocket-Protocol'] = ''
        bad_cases.append(
            ('Empty Sec-WebSocket-Protocol', request_def, 400, True))

        for (case_name, request_def, expected_status,
             expect_handshake_exception) in bad_cases:
            request = _create_request(request_def)
            handshaker = Handshaker(request, mock.MockDispatcher())
            try:
                handshaker.do_handshake()
                self.fail('No exception thrown for \'%s\' case' % case_name)
            except HandshakeException as e:
                self.assertTrue(expect_handshake_exception)
                self.assertEqual(expected_status, e.status)
            except VersionException as e:
                self.assertFalse(expect_handshake_exception)


if __name__ == '__main__':
    unittest.main()

# vi:sts=4 sw=4 et



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette_driver import By, Wait
from marionette_harness import MarionetteTestCase, WindowManagerMixin


class TestSSLStatusAfterRestart(WindowManagerMixin, MarionetteTestCase):
    def setUp(self):
        super(TestSSLStatusAfterRestart, self).setUp()
        self.marionette.set_context("chrome")

        self.test_url = "https://www.itisatrap.org/"

        # Set browser to restore previous session
        self.marionette.set_pref("browser.startup.page", 3)
        # Disable rcwn to make cache behavior deterministic
        self.marionette.set_pref("network.http.rcwn.enable", False)

    def tearDown(self):
        self.marionette.clear_pref("browser.startup.page")
        self.marionette.clear_pref("network.http.rcwn.enable")

        super(TestSSLStatusAfterRestart, self).tearDown()

    def test_ssl_status_after_restart(self):
        with self.marionette.using_context("content"):
            self.marionette.navigate(self.test_url)
        self.verify_certificate_status(self.test_url)

        self.marionette.restart(in_app=True)

        self.verify_certificate_status(self.test_url)

    def verify_certificate_status(self, url):
        with self.marionette.using_context("content"):
            Wait(self.marionette).until(
                lambda _: self.marionette.get_url() == url,
                message="Expected URL loaded",
            )

        identity_box = self.marionette.find_element(By.ID, "identity-box")
        self.assertEqual(identity_box.get_attribute("pageproxystate"), "valid")

        class_list = self.marionette.execute_script(
            """
          const names = [];
          for (const name of arguments[0].classList) {
            names.push(name);
          }
          return names;
        """,
            script_args=[identity_box],
        )
        self.assertIn("verifiedDomain", class_list)




import py
from py._xmlgen import unicode, html, raw
import sys

class ns(py.xml.Namespace):
    pass

def test_escape():
    uvalue = py.builtin._totext('\xc4\x85\xc4\x87\n\xe2\x82\xac\n', 'utf-8')
    class A:
        def __unicode__(self):
            return uvalue
        def __str__(self):
            x = self.__unicode__()
            if sys.version_info[0] < 3:
                return x.encode('utf-8')
            return x
    y = py.xml.escape(uvalue)
    assert y == uvalue
    x = py.xml.escape(A())
    assert x == uvalue
    if sys.version_info[0] < 3:
        assert isinstance(x, unicode)
        assert isinstance(y, unicode)
        y = py.xml.escape(uvalue.encode('utf-8'))
        assert y == uvalue


def test_tag_with_text():
    x = ns.hello("world")
    u = unicode(x)
    assert u == "<hello>world</hello>"

def test_class_identity():
    assert ns.hello is ns.hello

def test_tag_with_text_and_attributes():
    x = ns.some(name="hello", value="world")
    assert x.attr.name == 'hello'
    assert x.attr.value == 'world'
    u = unicode(x)
    assert u == '<some name="hello" value="world"/>'

def test_tag_with_subclassed_attr_simple():
    class my(ns.hello):
        class Attr(ns.hello.Attr):
            hello="world"
    x = my()
    assert x.attr.hello == 'world'
    assert unicode(x) == '<my hello="world"/>'

def test_tag_with_raw_attr():
    x = html.object(data=raw('&'))
    assert unicode(x) == '<object data="&"></object>'

def test_tag_nested():
    x = ns.hello(ns.world())
    unicode(x) # triggers parentifying
    assert x[0].parent is x
    u = unicode(x)
    assert u == '<hello><world/></hello>'

def test_list_nested():
    x = ns.hello([ns.world()]) #pass in a list here
    u = unicode(x)
    assert u == '<hello><world/></hello>'

def test_tag_xmlname():
    class my(ns.hello):
        xmlname = 'world'
    u = unicode(my())
    assert u == '<world/>'

def test_tag_with_text_entity():
    x = ns.hello('world & rest')
    u = unicode(x)
    assert u == "<hello>world &amp; rest</hello>"

def test_tag_with_text_and_attributes_entity():
    x = ns.some(name="hello & world")
    assert x.attr.name == "hello & world"
    u = unicode(x)
    assert u == '<some name="hello &amp; world"/>'

def test_raw():
    x = ns.some(py.xml.raw("<p>literal</p>"))
    u = unicode(x)
    assert u == "<some><p>literal</p></some>"


def test_html_name_stickyness():
    class my(html.p):
        pass
    x = my("hello")
    assert unicode(x) == '<p>hello</p>'

def test_stylenames():
    class my:
        class body(html.body):
            style = html.Style(font_size = "12pt")
    u = unicode(my.body())
    assert u == '<body style="font-size: 12pt"></body>'

def test_class_None():
    t = html.body(class_=None)
    u = unicode(t)
    assert u == '<body></body>'

def test_alternating_style():
    alternating = (
        html.Style(background="white"),
        html.Style(background="grey"),
    )
    class my(html):
        class li(html.li):
            def style(self):
                i = self.parent.index(self)
                return alternating[i%2]
            style = property(style)

    x = my.ul(
            my.li("hello"),
            my.li("world"),
            my.li("42"))
    u = unicode(x)
    assert u == ('<ul><li style="background: white">hello</li>'
                     '<li style="background: grey">world</li>'
                     '<li style="background: white">42</li>'
                 '</ul>')

def test_singleton():
    h = html.head(html.link(href="foo"))
    assert unicode(h) == '<head><link href="foo"/></head>'

    h = html.head(html.script(src="foo"))
    assert unicode(h) == '<head><script src="foo"></script></head>'

def test_inline():
    h = html.div(html.span('foo'), html.span('bar'))
    assert (h.unicode(indent=2) ==
            '<div><span>foo</span><span>bar</span></div>')

def test_object_tags():
    o = html.object(html.object())
    assert o.unicode(indent=0) == '<object><object></object></object>'



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import types

import six

import mozunit
import pytest

from marionette_harness.runner import serve
from marionette_harness.runner.serve import iter_proc, iter_url


def teardown_function(func):
    for server in [s for s in iter_proc(serve.servers) if s.is_alive]:
        server.stop()
        server.kill()


def test_registered_servers():
    # [(name, factory), ...]
    assert serve.registered_servers[0][0] == "http"
    assert serve.registered_servers[1][0] == "https"


def test_globals():
    assert serve.default_doc_root is not None
    assert serve.registered_servers is not None
    assert serve.servers is not None


def test_start():
    serve.start()
    assert len(serve.servers) == 2
    assert "http" in serve.servers
    assert "https" in serve.servers
    for url in iter_url(serve.servers):
        assert isinstance(url, six.string_types)


def test_start_with_custom_root(tmpdir_factory):
    tdir = tmpdir_factory.mktemp("foo")
    serve.start(str(tdir))
    for server in iter_proc(serve.servers):
        assert server.doc_root == tdir


def test_iter_proc():
    serve.start()
    for server in iter_proc(serve.servers):
        server.stop()


def test_iter_url():
    serve.start()
    for url in iter_url(serve.servers):
        assert isinstance(url, six.string_types)


def test_where_is():
    serve.start()
    assert serve.where_is("/") == serve.servers["http"][1].get_url("/")
    assert serve.where_is("/", on="https") == serve.servers["https"][1].get_url("/")


if __name__ == "__main__":
    mozunit.main("-p", "no:terminalreporter", "--log-tbpl=-", "--capture", "no")
